package com.jzlife.report.compent;

import java.io.BufferedReader;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.LinkedHashMap;
import java.util.LinkedList;
import java.util.List;
import java.util.Map;

import com.alibaba.excel.context.AnalysisContext;
import com.alibaba.excel.event.AnalysisEventListener;
import com.jzlife.report.constant.ImportExcelConstant;
import com.jzlife.report.entity.reports.ImportDataRecord;
import com.jzlife.report.service.newreport.ServicePowerReportService;
import com.jzlife.report.service.report.FileScannerReadExcelService;
import com.jzlife.report.util.SpringContextUtils;

import lombok.extern.slf4j.Slf4j;

@Slf4j
public class EasyExcelListener extends AnalysisEventListener {

	// 设置最大数
	private static final int BATCH_COUNT = 1000;
	private static int COUT = 0;// 总数
	private ImportDataRecord dr;// import_data_record的id

	// 临时缓存List
	public static List<List<String>> importList = new ArrayList();

	public EasyExcelListener(ImportDataRecord dr) {
		this.dr = dr;
	}

	@Override
	public void doAfterAllAnalysed(AnalysisContext arg0) {
		// 解析完毕需要做的功能 这里是解析完毕存入数据库 清空当前List
		log.info("导入结束了");
		if (importList.size() > 0) {
			// insert数据或调用逻辑，
			if (dr.getTypeCode().equals(ImportExcelConstant.TYPE_CODE_1002)) {
				ServicePowerReportService servicePowerReportService = SpringContextUtils
						.getBeanByClass(ServicePowerReportService.class);
				servicePowerReportService.import30DayProductStockData(importList);
			}
			importList.clear();
		}

		FileScannerReadExcelService fileScanner = SpringContextUtils.getBeanByClass(FileScannerReadExcelService.class);
		fileScanner.updateComplate(this.dr.getId(), ImportExcelConstant.STATE_SUCCESS, COUT, null);
	}

	@Override
	public void invoke(Object entity, AnalysisContext arg1) {
		if (entity == null) {
			return;
		}

		LinkedHashMap<String, String> map = (LinkedHashMap) entity;
		importList.add(new LinkedList(map.values()));

		if (importList.size() >= BATCH_COUNT) {
			if (dr.getTypeCode().equals(ImportExcelConstant.TYPE_CODE_1002)) {
				ServicePowerReportService servicePowerReportService = SpringContextUtils
						.getBeanByClass(ServicePowerReportService.class);
				servicePowerReportService.import30DayProductStockData(importList);
			}
			importList.clear();
		}
		COUT++;
	}

	/**
	 * csv方式导入
	 * @param inputStream
	 * @param readListener
	 * @throws Exception
	 */
	public static void importCsv(InputStream inputStream,
			AnalysisEventListener<LinkedHashMap<Integer, String>> readListener) throws Exception {
		BufferedReader reader = new BufferedReader(new InputStreamReader(inputStream, "GBK"));

		int count = 0;
		LinkedHashMap<Integer, String> headMap = new LinkedHashMap<Integer, String>();
		while (true) {
			count++;
			String s = reader.readLine();
			if (s != null) {
				// 数据头
				if (count == 1) {
					String[] split = s.split(",");
					for (int i = 0; i < split.length; i++) {
						headMap.put(i, split[i]);// specialUnicode(split[i])
					}
					readListener.invokeHeadMap(headMap, null);
				} else {
					String[] split = s.split(",");
					LinkedHashMap<Integer, String> entity = new LinkedHashMap<Integer, String>();
					for (int i = 0; i < split.length; i++) {
						entity.put(i, clearStartAndEndQuote(split[i]));
					}
					readListener.invoke(entity, null);
				}
			} else {
				break;
			}
		}

		readListener.doAfterAllAnalysed(null);
	}

	/**
	 * 去除 字符串收尾的 特殊的Unicode [ "\uFEFF" ] csv 文件可能会带有该编码
	 * 
	 * @param str
	 * @return
	 */
	public static String specialUnicode(String str) {
		if (str.startsWith("\uFEFF")) {
			str = str.replace("\uFEFF", "");
		} else if (str.endsWith("\uFEFF")) {
			str = str.replace("\uFEFF", "");
		}
		return str;
	}

	// 去掉字符串前后字符串“” for csv输出
	public static String clearStartAndEndQuote(String str) {
		if (str != null && str.length() >= 2) {
			if (str.indexOf("\"") == 0)
				str = str.substring(1, str.length()); // 去掉第一个 "
			if (str.lastIndexOf("\"") == (str.length() - 1))
				str = str.substring(0, str.length() - 1); // 去掉最后一个 "

			str = str.replaceAll("\"\"", "\"");// 把两个双引号换成一个双引号
		}
		return str;
//		return specialUnicode(str);
	}

	public static void main(String[] args) {
		String value = "\"1234\"";

		System.out.println(clearStartAndEndQuote(value));
	}

}
