package com.jzlife.report.config.mvc;


import com.alibaba.fastjson.JSONArray;
import com.alibaba.fastjson.JSONObject;
import com.jzlife.report.exception.LoginException;
import com.jzlife.report.service.redis.RedisService;
import com.jzlife.report.util.JsonUtil;
import io.swagger.models.HttpMethod;
import lombok.extern.slf4j.Slf4j;
import org.springframework.web.context.WebApplicationContext;
import org.springframework.web.context.support.WebApplicationContextUtils;
import org.springframework.web.servlet.HandlerExceptionResolver;

import javax.servlet.*;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.util.Map;


@Slf4j
public class BodyReaderRequestFilter implements Filter {


    private RedisService redisService;

    private HandlerExceptionResolver resolver;


    @Override
    public void init(FilterConfig filterConfig) {
        ServletContext context = filterConfig.getServletContext();
        WebApplicationContext applicationContext = WebApplicationContextUtils.getWebApplicationContext(context);
        if (redisService == null) {
            assert applicationContext != null;
            redisService = applicationContext.getBean(RedisService.class);
            resolver = applicationContext.getBean("handlerExceptionResolver", HandlerExceptionResolver.class);
        }
    }


    @Override
    public void doFilter(ServletRequest req, ServletResponse res, FilterChain filterChain) throws IOException, ServletException {
        HttpServletRequest request = (HttpServletRequest) req;
        HttpServletResponse response = (HttpServletResponse) res;
        RequestWrapper requestWrapper = new RequestWrapper(request);
        String body = requestWrapper.getBody();
        String url = request.getRequestURI();

        log.info("当前执行的地址是：" + url);
        log.info("当前执行的body参数是：" + body);
        Map<String, String[]> parameterMap = request.getParameterMap();
        log.info("当前执行的query参数是" + JsonUtil.getJson(parameterMap));
        log.info("---------------------------------------");

        if (url.contains("/login") | url.contains("error") | url.contains("syncMail")||url.contains("swagger")||url.contains("api-docs")) {
            filterChain.doFilter(requestWrapper, res);
            return;
        }
        try {
            String token = request.getHeader("token");
            if (null == token) {
                token = request.getParameter("token");
            }
            if (null == token) {
                throw new LoginException("登陆过时，或未登陆");
            }
            String obj = redisService.get(token);
            if (null != obj) {
                try {
                    JSONObject o = (JSONObject) JSONArray.parseArray(obj).get(1);
                    log.info(o.getString("userAccount") + "用户访问了" + url);
                    requestWrapper.setAttribute("userId", o.getInteger("id"));
                } catch (Exception e) {
                    throw new LoginException("账号权限校验失败");
                }
            } else {
                throw new LoginException("账号权限校验失败");
            }
            if (HttpMethod.POST.toString().matches(requestWrapper.getMethod())) {
                handlePostRequest(requestWrapper);
            }
            if (HttpMethod.GET.toString().matches(requestWrapper.getMethod())) {
                handleGetRequest(requestWrapper);
            }
            filterChain.doFilter(requestWrapper, response);
        } catch (Exception e) {
            if (e instanceof IllegalStateException) {
                log.info("请求参数异常");
            }
            resolver.resolveException(requestWrapper, response, null, e);
        }

    }

    protected void handleGetRequest(RequestWrapper requestWrapper) {
        String queryString = requestWrapper.getQueryString();
        if (queryString != null) {
            requestWrapper.setBody(queryString);
        }
    }

    protected void handlePostRequest(RequestWrapper requestWrapper) {
        String body = requestWrapper.getBody();
        if (body != null) {
            requestWrapper.setBody(body);
        }
    }

    @Override
    public void destroy() {

    }


}
