package com.jzlife.report.controller;

import com.alibaba.fastjson.JSONObject;
import com.google.common.collect.Lists;
import com.jzlife.report.anno.LogRecordFiled;
import com.jzlife.report.dao.douyin.DouyinBoardMapper;
import com.jzlife.report.dto.EchartsLine;
import com.jzlife.report.dto.douyin.DouyinBoardBookVo;
import com.jzlife.report.dto.douyin.DouyinBoardLineVo;
import com.jzlife.report.dto.douyin.DouyinBoardMapVo;
import com.jzlife.report.dto.douyin.DouyinBoardSalesVo;
import com.jzlife.report.service.redis.RedisService;
import com.jzlife.report.util.DateUtil;
import com.jzlife.report.util.DateUtils;
import com.jzlife.report.util.ResponseUtil;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import java.time.Duration;
import java.time.LocalDateTime;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

/**
 * 抖音看板相关报表
 */
@RestController
@RequestMapping("douyinBoard")
public class DouyinBoardController {

    @Resource
    private DouyinBoardMapper boardMapper;

    @Autowired
    private RedisService redisService;

    @ApiOperation(value = "抖音看板相关报表", httpMethod = "GET", notes = "获取今日销量数据")
    @GetMapping("today")
    @LogRecordFiled(funName = "获取今日销量数据")
    public Object getToday(String channel) throws Exception {
        Map<String, Object> obj = new HashMap<String, Object>();
        obj.put("refreshTime", LocalDateTime.now());

        String redisKey = "Day_DouyinBoardSalesVo" + channel;

        DouyinBoardSalesVo boardBookVo;
        String value = redisService.get(redisKey);
        if (StringUtils.isNotEmpty(value))
            boardBookVo = JSONObject.parseObject(value, DouyinBoardSalesVo.class);
        else {
            boardBookVo = boardMapper.getTodaySales(channel);
            Duration duration = Duration.between(LocalDateTime.now(), LocalDateTime.now().plusMinutes(1));
            redisService.set(redisKey, JSONObject.toJSONString(boardBookVo), duration);
        }

        obj.put("data", boardBookVo);
        return ResponseUtil.ok(obj);
    }

    @ApiOperation(value = "抖音看板相关报表", httpMethod = "GET", notes = "获取当月销量数据")
    @GetMapping("month")
    @LogRecordFiled(funName = "获取当月销量数据")
    public Object getMonth(String channel) throws Exception {
        Map<String, Object> obj = new HashMap<String, Object>();
        obj.put("refreshTime", LocalDateTime.now());

        String redisKey = "Month_DouyinBoardSalesVo" + channel;

        DouyinBoardSalesVo monthBoardBookVo;
        String value = redisService.get(redisKey);
        if (StringUtils.isNotEmpty(value))
            monthBoardBookVo = JSONObject.parseObject(value, DouyinBoardSalesVo.class);
        else {
            monthBoardBookVo = boardMapper.getMonthSales(channel);
            Duration duration = Duration.between(LocalDateTime.now(), LocalDateTime.now().plusMinutes(10));
            redisService.set(redisKey, JSONObject.toJSONString(monthBoardBookVo), duration);
        }

        obj.put("data", monthBoardBookVo);

        return ResponseUtil.ok(obj);
    }

    @ApiOperation(value = "抖音看板相关报表", httpMethod = "GET", notes = "获取销售热力图")
    @GetMapping("hotMap")
    @LogRecordFiled(funName = "获取销售热力图")
    public Object getHotMap(Integer days,String channel) {
        //List<DouyinBoardMapVo> list = boardMapper.getHotMap(days,channel);

        String redisKey = "DouyinBoardMapVo" + channel+days.toString();

        List<DouyinBoardMapVo> list;
        String value = redisService.get(redisKey);
        if (StringUtils.isNotEmpty(value))
            list = JSONObject.parseArray(value, DouyinBoardMapVo.class);
        else {
            list = boardMapper.getHotMap(days,channel);
            Duration duration = Duration.between(LocalDateTime.now(), LocalDateTime.now().plusMinutes(10));
            redisService.set(redisKey, JSONObject.toJSONString(list), duration);
        }

        List<String> result = Lists.newArrayList();
        list.forEach(v -> {
            if (StringUtils.isNotEmpty(v.getPosition()) && v.getPosition().equals(",") == false)
                result.add(v.getPosition());
            else {
                result.add(getCityLnglat(v.getCity()));
            }
        });
        return ResponseUtil.ok(result);
    }

    @ApiOperation(value = "抖音看板相关报表", httpMethod = "GET", notes = "近30天销量及服务情况")
    @GetMapping("lineData")
    @LogRecordFiled(funName = "近30天销量及服务情况")
    public Object lineData(String channel) throws Exception {
        //定义4条曲线
        EchartsLine echOrdersList = new EchartsLine("line");//订单量
        echOrdersList.setName("销量");
        echOrdersList.setData(Lists.newArrayList());

        EchartsLine echRefundList = new EchartsLine("line");//退单量
        echRefundList.setName("退单量");
        echRefundList.setData(Lists.newArrayList());

        EchartsLine echServicedList = new EchartsLine("line");//已服务量
        echServicedList.setName("已服务订单");
        echServicedList.setData(Lists.newArrayList());

        EchartsLine echunbookedList = new EchartsLine("line");//未预约
        echunbookedList.setName("未预约订单");
        echunbookedList.setData(Lists.newArrayList());

        EchartsLine matchingList = new EchartsLine("line");//匹配中
        matchingList.setName("匹配中");
        matchingList.setData(Lists.newArrayList());

        EchartsLine dGmv = new EchartsLine("bar");//直营
        dGmv.setName("直营GMV");
        dGmv.setStack("Total2");
        dGmv.setData(Lists.newArrayList());

        EchartsLine gmv = new EchartsLine("bar");//gmv
        gmv.setName("GMV");
        gmv.setStack("Total");
        gmv.setData(Lists.newArrayList());


        String redisKey = "DouyinBoardLineVo" + channel;

        List<DouyinBoardLineVo> list;
        String value = redisService.get(redisKey);
        if (StringUtils.isNotEmpty(value))
            list = JSONObject.parseArray(value, DouyinBoardLineVo.class);
        else {
            list = boardMapper.getLineData(channel);
            Duration duration = Duration.between(LocalDateTime.now(), LocalDateTime.now().plusMinutes(60));
            redisService.set(redisKey, JSONObject.toJSONString(list), duration);
        }



        List<String> dayList = Lists.newArrayList();
        //近30天日期格式化
        for (int i = 30; i > 0; i--) {
            String date = DateUtils.format.format(DateUtil.addDay(new Date(), -i));
            dayList.add(date);

            List<DouyinBoardLineVo> dList = list.stream().filter(item -> item.getDate().equals(date)).collect(Collectors.toList());
            echOrdersList.getData().add(dList.size() > 0 ? dList.get(0).getTotal() : 0);
            echRefundList.getData().add(dList.size() > 0 ? dList.get(0).getRefunds() : 0);
            echServicedList.getData().add(dList.size() > 0 ? dList.get(0).getServiced() : 0);
            echunbookedList.getData().add(dList.size() > 0 ? dList.get(0).getUnbooked() : 0);
            matchingList.getData().add(dList.size() > 0 ? dList.get(0).getMatching() : 0);
            dGmv.getData().add(dList.size() > 0 ? dList.get(0).getDGmv() : 0);
            gmv.getData().add(dList.size() > 0 ? dList.get(0).getGmv() : 0);
        }

        List<EchartsLine> echLiss = Lists.newArrayList();
        echLiss.add(echOrdersList);
        echLiss.add(echRefundList);
        echLiss.add(echServicedList);
        echLiss.add(echunbookedList);
        echLiss.add(gmv);
        echLiss.add(dGmv);

        Map<String, Object> data = new HashMap<String, Object>();
        data.put("series", echLiss);
        data.put("days", dayList);

        return ResponseUtil.ok(data);
    }

    @ApiOperation(value = "抖音看板相关报表", httpMethod = "GET", notes = "近30天每日服务情况")
    @GetMapping("lineBookData")
    @LogRecordFiled(funName = "近30天每日服务情况")
    public Object lineBookData(String channel) throws Exception {
        //定义4条曲线
        EchartsLine echOrdersList = new EchartsLine("line");//订单量
        echOrdersList.setName("联营");
        echOrdersList.setData(Lists.newArrayList());

        EchartsLine echRefundList = new EchartsLine("line");//退单量
        echRefundList.setName("直营");
        echRefundList.setData(Lists.newArrayList());

        String redisKey = "DouyinBoardBookVo" + channel;

        List<DouyinBoardBookVo> list ;//= boardMapper.getServiceBookDataForLine(channel)
        String value = redisService.get(redisKey);
        if (StringUtils.isNotEmpty(value))
            list = JSONObject.parseArray(value, DouyinBoardBookVo.class);
        else {
            list = boardMapper.getServiceBookDataForLine(channel);
            Duration duration = Duration.between(LocalDateTime.now(), LocalDateTime.now().plusMinutes(60));
            redisService.set(redisKey, JSONObject.toJSONString(list), duration);
        }

        List<String> dayList = Lists.newArrayList();
        //近30天日期格式化
        for (int i = 30; i > 0; i--) {
            String date = DateUtils.format.format(DateUtil.addDay(new Date(), -i));
            dayList.add(date);

            List<DouyinBoardBookVo> dList = list.stream().filter(item -> item.getDate().equals(date)).collect(Collectors.toList());
            echOrdersList.getData().add(dList.size() > 0 ? dList.get(0).getFraCount() : 0);
            echRefundList.getData().add(dList.size() > 0 ? dList.get(0).getDirectCount() : 0);
        }

        List<EchartsLine> echLiss = Lists.newArrayList();
        echLiss.add(echOrdersList);
        echLiss.add(echRefundList);

        Map<String, Object> data = new HashMap<String, Object>();
        data.put("series", echLiss);
        data.put("days", dayList);

        return ResponseUtil.ok(data);
    }

    String getCityLnglat(String city) {
        city = city == null ? "上海" : city.replace("市", "");
        switch (city) {
            case "北京":
                return "116.720996, 39.903666";
            case "深圳":
                return "114.05522, 22.542207";
            case "南京":
                return "118.665131, 31.930355";
            case "苏州":
                return "120.58053, 31.302569";
            case "杭州":
                return "120.20805, 30.247872";
            case "重庆":
                return "106.511589,29.532241";
            case "成都":
                return "104.065521,30.657529";
            case "青岛":
                return "120.371836,36.084444";
            case "昆山":
                return "121.074918,31.306858";
            case "广州":
                return "113.253725,23.12607";
            default://默认上海
                return "121.47102, 31.229203";
        }
    }



}
