package com.jzlife.report.controller;

import java.io.IOException;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import com.alibaba.excel.EasyExcel;
import com.jzlife.report.anno.LogRecordFiled;
import com.jzlife.report.dto.excelimport.HmScheduleExcel;
import com.jzlife.report.dto.excelimport.HmScheduleOnleaveExcel;
import com.jzlife.report.entity.hmorder.ReportEvaluation;
import com.jzlife.report.entity.hmorder.ReportRebuy;
import com.jzlife.report.entity.hmorder.ReportSignIn;
import com.jzlife.report.schedule.ImportExcelTask;
import com.jzlife.report.service.hmorder.ImportExcelService;
import com.jzlife.report.util.ExcelListener;
import com.jzlife.report.util.ResponseUtil;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import lombok.extern.slf4j.Slf4j;

@Slf4j
@Api(value = "导入功能", description = "导入功能")
@RestController
@RequestMapping("importExcel")
public class ImportExcelController {

	@Autowired
	ImportExcelService importExcelService;
	
	@Autowired
	ImportExcelTask importExcelTask;
	
	@ApiOperation("手动跑导入数据")
	@GetMapping("/runImportTask")
	public Object runImportTask() {
		
		importExcelTask.runImportTask();
		  new Thread(() -> {
			  importExcelTask.runImportTask();
	        }).start();
		return ResponseUtil.ok();
	}

	@ApiOperation("生活服务评价导入")
	@PostMapping("/evaluateLifeService")
	@LogRecordFiled(funName = "生活服务评价导入")
	public Object evaluateLifeService(@RequestParam(value = "file") MultipartFile file) throws IOException {

		ExcelListener<ReportEvaluation> listener = new ExcelListener<ReportEvaluation>();
		EasyExcel.read(file.getInputStream(), ReportEvaluation.class, listener).sheet(0).headRowNumber(1).doReadSync();

		List<ReportEvaluation> datas = listener.getDatas();
		log.info("生活服务评价导入 数据条数:" + datas.size());

		importExcelService.insertReportEvaluation(datas);

		return ResponseUtil.ok();
	}

	@ApiOperation("复购导入")
	@PostMapping("/rebuy")
	@LogRecordFiled(funName = "复购导入")
	public Object rebuy(@RequestParam(value = "file") MultipartFile file) throws IOException {

		ExcelListener<ReportRebuy> listener = new ExcelListener<ReportRebuy>();
		EasyExcel.read(file.getInputStream(), ReportRebuy.class, listener).sheet(0).headRowNumber(1).doReadSync();

		List<ReportRebuy> datas = listener.getDatas();
		log.info("复购导入  数据条数:" + datas.size());

		importExcelService.insertReportRebuy(datas);

		return ResponseUtil.ok();
	}

	@ApiOperation("打卡记录导入")
	@PostMapping("/clockLog")
	@LogRecordFiled(funName = "打卡记录导入")
	public Object clockLog(@RequestParam(value = "file") MultipartFile file) throws IOException {

		ExcelListener<ReportSignIn> listener = new ExcelListener<ReportSignIn>();
		EasyExcel.read(file.getInputStream(), ReportSignIn.class, listener).sheet(0).headRowNumber(1).doReadSync();

		List<ReportSignIn> datas = listener.getDatas();
		log.info("打卡记录导入  数据条数:" + datas.size());

		importExcelService.insertReportSignIn(datas);

		return ResponseUtil.ok();
	}
	
	//盒马排班 ,盒马请假
	
	@ApiOperation("盒马排班，盒马请假导入，fileScheduler排班，fileOnleave请假")
	@PostMapping("/scheduleAndOnleave")
	@LogRecordFiled(funName = "盒马排班，盒马请假导入")
	public Object scheduleAndOnleave(@RequestParam(value = "fileScheduler") MultipartFile fileScheduler,
			@RequestParam(value = "fileOnleave") MultipartFile fileOnleave) throws IOException {
		
		ExcelListener<HmScheduleExcel> scheduler = new ExcelListener<HmScheduleExcel>();
		EasyExcel.read(fileScheduler.getInputStream(), HmScheduleExcel.class, scheduler).sheet(0).headRowNumber(1).doReadSync();
		List<HmScheduleExcel> schedulerList = scheduler.getDatas();
		
		ExcelListener<HmScheduleOnleaveExcel> leave = new ExcelListener<HmScheduleOnleaveExcel>();
		EasyExcel.read(fileOnleave.getInputStream(), HmScheduleOnleaveExcel.class, leave).sheet(0).headRowNumber(1).doReadSync();
		List<HmScheduleOnleaveExcel> leaveList = leave.getDatas();
 
		System.out.println("schedulerList:"+schedulerList.size());
		System.out.println("leaveList:"+leaveList.size());
 
		importExcelService.insertScheduleAndOnleave(schedulerList,leaveList);

		return ResponseUtil.ok();
	}
	
	

	
}
