package com.jzlife.report.controller;


import cn.afterturn.easypoi.excel.entity.ExportParams;
import com.github.pagehelper.PageInfo;
import com.google.common.collect.Lists;
import com.jzlife.report.anno.LogRecordFiled;
import com.jzlife.report.dto.export.BjSettlementExportDto;
import com.jzlife.report.excelExport.DayInvestMenDetail;
import com.jzlife.report.excelExport.numberConvert.ExcelStyleUtil;
import com.jzlife.report.service.newreport.InvestmentReportService;
import com.jzlife.report.util.ExcelUtils;
import com.jzlife.report.util.ResponseUtil;
import com.jzlife.report.vo.DayReportBaseVo;
import com.jzlife.report.vo.InvestmentAndResignVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import io.swagger.annotations.ApiParam;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.time.LocalDate;
import java.time.format.DateTimeFormatter;
import java.util.Date;
import java.util.List;
import java.util.Map;

@Api(value = "招商数据看板", description = "招商数据看板")
@RestController
@RequestMapping(value = "report")
public class InvestmentReportController {

    @Autowired
    private InvestmentReportService investmentReportService;


    @ApiOperation(value = "招商情况和离职情况及各来源入职情况", httpMethod = "POST", notes = "时间格式为：yyyy-MM-dd HH:mm:ss")
    @PostMapping("investmentAndResignList")
    @LogRecordFiled(funName = "招商看板")
    public Object investmentAndResignList(@Validated @RequestBody InvestmentAndResignVo vo, HttpServletRequest request) {
        String errorMsg = checkChooseTime(vo.getStartTime(), vo.getEndTime());
        if (StringUtils.isNotBlank(errorMsg)) {
            return ResponseUtil.fail400(errorMsg);
        }
        if (StringUtils.isEmpty(vo.getCity())) {
            String citys = request.getHeader("allowCity");
            if (StringUtils.isNotEmpty(citys)) {
                vo.setCity(citys);
            }
        }
        Map<String, Object> data = investmentReportService.investmentAndResignList(vo);
        return ResponseUtil.ok(data);
    }

    private String checkChooseTime(String startTime, String endTime) {
        LocalDate start = LocalDate.parse(startTime, DateTimeFormatter.ofPattern("yyyy-MM-dd"));
        LocalDate end = LocalDate.parse(endTime, DateTimeFormatter.ofPattern("yyyy-MM-dd"));
        if (start.isAfter(end) || start.isEqual(end)) {
            return "开始时间不可大于或等于结束时间！";
        }
        if (end.getYear() - start.getYear() > 1) {
            return "所选时间跨度不可超过1年";
        }
        if (start.isAfter(LocalDate.now().minusDays(1))) {
            return "仅支持查看T-1数据";
        }
        return "";
    }


    @ApiOperation(value = "查看离职&跟单流失原因", httpMethod = "GET", notes = "查看离职&跟单流失原因")
    @GetMapping("getResignAndLossReason")
    @LogRecordFiled(funName = "离职&跟单流失看板")
    public Object getResignAndLossReason(@RequestParam(value = "type", defaultValue = "1,2")
                                         @ApiParam(name = "type", value = "查询类型  1离职 2流失", defaultValue = "1,2") String type,
                                         @ApiParam(name = "startTime", value = "yyyy-MM-dd") @RequestParam("startTime") String startTime,
                                         @ApiParam(name = "endTime", value = "yyyy-MM-dd") @RequestParam("endTime") String endTime,
                                         @ApiParam(name = "city", value = "城市") @RequestParam(value = "city", required = false) String city,
                                         @ApiParam(name = "postStation", value = "驿站") @RequestParam(value = "postStation", required = false) String postStation,
                                         @ApiParam(name = "serviceStation", value = "服务站") @RequestParam(value = "serviceStation", required = false) String serviceStation,
                                         HttpServletRequest request
    ) {
        if (StringUtils.isEmpty(city)) {
            String citys = request.getHeader("allowCity");
            if (StringUtils.isNotEmpty(citys)) {
                city = citys;
            }
        }
        return ResponseUtil.ok(investmentReportService.getResignAndLossReason(type, startTime, endTime, city, postStation, serviceStation));
    }


    @ApiOperation(value = "招商明细", httpMethod = "POST", notes = "时间格式为：yyyy-MM-dd,查询选定时间段内的每日招商明细")
    @PostMapping("dayInvestMenDetail")
    @LogRecordFiled(funName = "招商明细")
    public Object dayInvestMenDetail(@Validated @RequestBody DayReportBaseVo vo, HttpServletRequest request) {
        if (StringUtils.isEmpty(vo.getCity())) {
            String citys = request.getHeader("allowCity");
            if (StringUtils.isNotEmpty(citys)) {
                vo.setCity(citys);
            }
        }
        PageInfo<Map<String, Object>> data = investmentReportService.dayInvestMentDetail(vo);
        return ResponseUtil.ok(data);
    }

    @ApiOperation(value = "导出招商明细", httpMethod = "GET", notes = "时间格式为：yyyy-MM-dd,导出选定时间段内的每日招商明细")
    @GetMapping("exportDayInvestMenDetail")
    @LogRecordFiled(funName = "导出招商明细")
    public void exportDayInvestMenDetail(
            String city,
            @RequestParam("startTime") String startTime,
            @RequestParam("endTime") String endTime,
            HttpServletRequest request,
            HttpServletResponse response) throws Exception {
        DayReportBaseVo vo = new DayReportBaseVo();
        vo.setCity(city);
        vo.setStartTime(startTime);
        vo.setEndTime(endTime);
        if (StringUtils.isEmpty(vo.getCity())) {
            String citys = request.getHeader("allowCity");
            if (StringUtils.isNotEmpty(citys)) {
                vo.setCity(citys);
            }
        }
        List<Map<String, Object>> data = investmentReportService.dayInvestMentDetailExport(vo);
        ExportParams exportParams = new ExportParams();
        exportParams.setStyle(ExcelStyleUtil.class);
        exportParams.setTitle("招商明细");
        exportParams.setSheetName("招商明细");
        String fileName = "招商明细-" + vo.getStartTime() + "至" + vo.getEndTime();
        ExcelUtils.defaultExport(data(data), DayInvestMenDetail.class, fileName, response, exportParams);
    }

    private List<DayInvestMenDetail> data(List<Map<String, Object>> data) {
        List<DayInvestMenDetail> dayInvestMenDetails = Lists.newArrayList();
        for (Map<String, Object> objectMap : data) {
            DayInvestMenDetail dayInvestMenDetail = new DayInvestMenDetail();
            dayInvestMenDetail.setDate((Date) objectMap.get("日期"));
            dayInvestMenDetail.setCity(objectMap.get("城市"));
            dayInvestMenDetail.setPublicSeaInput(objectMap.get("公海录入"));
            dayInvestMenDetail.setPrivateSeaInput(objectMap.get("私海录入"));
            dayInvestMenDetail.setPartnerInput(Math.toIntExact((Long) objectMap.get("合伙人录入")));
            dayInvestMenDetail.setTraining(Math.toIntExact((Long) objectMap.get("参培人数")));
            dayInvestMenDetail.setTrainPass(Math.toIntExact((Long) objectMap.get("培训通过")));
            dayInvestMenDetail.setWaitDocumentary(Math.toIntExact((Long) objectMap.get("等待跟单")));
            dayInvestMenDetail.setDocumentary(Math.toIntExact((Long) objectMap.get("进入跟单")));
            dayInvestMenDetail.setDocumentaryNoPass(Math.toIntExact((Long) objectMap.get("跟单不通过")));
            dayInvestMenDetail.setDocumentaryPass(Math.toIntExact((Long) objectMap.get("跟单通过")));
            dayInvestMenDetails.add(dayInvestMenDetail);
        }
        return dayInvestMenDetails;
    }

    @ApiOperation(value = "保洁结算", httpMethod = "POST", notes = "ofMonth格式为：yyyy-MM")
    @GetMapping("bjSettlement")
    @LogRecordFiled(funName = "保洁结算")
    public Object bjSettlement(HttpServletRequest request, HttpServletResponse response,
                               @RequestParam(defaultValue = "false") Boolean isExport,
                               @RequestParam(defaultValue = "1") Integer currentPage,
                               @RequestParam(defaultValue = "50") Integer pageSize,
                               String ofMonth, String store, String workerName) throws Exception {

        if (StringUtils.isBlank(ofMonth)) {
            return ResponseUtil.fail("月份不能为空");
        }

        Object res = investmentReportService.bjSettlement(currentPage, pageSize, isExport, ofMonth, store, workerName);
        if (isExport) {
            List<BjSettlementExportDto> list = (List<BjSettlementExportDto>) res;
            if (list != null && list.size() > 0) {
                ExcelUtils.exportExcel(list, null, "Sheet1", BjSettlementExportDto.class, "保洁结算.xls", response);
            }
        }
        return ResponseUtil.ok(res);
    }

    @ApiOperation(value = "招商结算", notes = "ofMonth格式为：yyyy-MM")
    @GetMapping("bmSettlement")
    @LogRecordFiled(funName = "招商结算")
    public Object bmSettlement(HttpServletRequest request, HttpServletResponse response,
                               String ofMonth, Boolean isExport) throws Exception {

        if (StringUtils.isBlank(ofMonth)) {
            return ResponseUtil.fail("月份不能为空");
        }

        return investmentReportService.bmSettlement(ofMonth, isExport, response);
    }

}
