package com.jzlife.report.controller;

import com.jzlife.report.anno.LogRecordFiled;
import com.jzlife.report.dao.reports.SPMapper;
import com.jzlife.report.dto.*;
import com.jzlife.report.service.newreport.MarketReportService;
import com.jzlife.report.service.nurse.CityStationService;
import com.jzlife.report.util.ExcelUtils;
import com.jzlife.report.util.ResponseUtil;
import com.jzlife.report.util.UserUtil;
import com.jzlife.report.util.page.BeanUtil;
import com.jzlife.report.util.page.PagedResult;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.List;

/**
 * 销售复购提醒报表 by leo
 *
 * 市场部销售看板 报表相关
 */
@RestController
@RequestMapping(value = "market")
public class MarketReportController {

    @Autowired
    private MarketReportService reportService;

    @Autowired
    UserUtil userUtil;

    @Autowired
    CityStationService cityStationService;

    @Autowired
    SPMapper spMapper;

    @ApiOperation("销售复购提醒")
    @GetMapping("remind")
    @LogRecordFiled(funName = "销售复购提醒")
    public Object remind(@RequestParam(defaultValue = "1") Integer pageNo,
                         @RequestParam(defaultValue = "50") Integer pageSize, String city, String bookDate, @RequestParam(defaultValue = "false") Boolean isExport,
                         HttpServletResponse response, HttpServletRequest request) throws Exception {

        if (StringUtils.isBlank(city)) {
            return ResponseUtil.fail("请选择城市");
        }

        if (isExport == false) {
            List<ReportMarketRebuyRemind> result = reportService.getMarketRebuyRemindByCity(pageNo, pageSize, city, bookDate);
            return ResponseUtil.ok(result);
        } else {
            List<ReportMarketRebuyRemind> result = reportService.getMarketRebuyRemindByCity(1, 10000, city, bookDate);
            ExcelUtils.exportExcel(result, null, "Sheet1", ReportMarketRebuyRemind.class, "小程序订单列表.xls",
                    response);
            return null;
        }
    }


    @ApiOperation("新客复购率报表")
    @GetMapping("ncRebuy")
    @LogRecordFiled(funName = "新客复购率报表")
    public Object ncRebuy(@RequestParam(defaultValue = "false") Boolean isExport,
                          String month,
                          String rebuy,
                          String cType,
                          HttpServletResponse response, HttpServletRequest request) throws Exception {

        List<ReportNewCustomerRebuy> list = reportService.getNewCustomerRebuyList(month, cType, rebuy);
        if (isExport == false)
            return ResponseUtil.ok(list);
        else {
            ExcelUtils.exportExcel(list, null, "Sheet1", ReportNewCustomerRebuy.class, "新客复购率报表.xls", response);
            return null;
        }
    }

    @ApiOperation("保洁品类新客量")
    @GetMapping("newcustomer")
    @LogRecordFiled(funName = "保洁品类新客量")
    public Object newcustomer(@RequestParam(defaultValue = "false") Boolean isExport,
                              @RequestParam(defaultValue = "1") Integer pageNo,
                              @RequestParam(defaultValue = "50") Integer pageSize,
                              String month,
                              HttpServletResponse response) throws Exception {

        List<ReportSalesNewcustomerV3> list = reportService.getNewCustomer(month,pageNo,pageSize);
        if (isExport == false) {
            PagedResult<ReportSalesNewcustomerV3> pagedResult = BeanUtil.toPagedResult(list);
            return ResponseUtil.ok(pagedResult);
        }
        else {
            ExcelUtils.exportExcel(list, null, "Sheet1", ReportSalesNewcustomerV3.class, "新客复购率报表.xls", response);
            return null;
        }
    }

    @ApiOperation("老客续约率")
    @GetMapping("ocRebuy")
    @LogRecordFiled(funName = "老客续约率：周期卡内最后一次服务完成后30天内，有未使用的、未预约的、履约中的订单")
    public Object ocRebuy(@RequestParam(defaultValue = "false") Boolean isExport,
                              String month,
                              HttpServletResponse response) throws Exception {

        List<ReportOldCustomerRebuy> list = reportService.getOldCustomerRebuy(month);
        if (isExport == false) {
            return ResponseUtil.ok(list);
        }
        else {
            ExcelUtils.exportExcel(list, null, "Sheet1", ReportOldCustomerRebuy.class, "老客续约率.xls", response);
            return null;
        }
    }

    @ApiOperation("保洁新客首次履约后T+7复购率")
    @GetMapping("ncRebuyTotal")
    @LogRecordFiled(funName = "保洁新客首次履约后T+7复购周期卡")
    public Object ncRebuyTotal(@RequestParam(defaultValue = "false") Boolean isExport,
                          String month,
                          HttpServletResponse response) throws Exception {

        List<ReportNewCustomerRebuyBJTotal> list = reportService.serviceNewCustomerRebuyBJTotal(month);
        if (isExport == false) {
            return ResponseUtil.ok(list);
        }
        else {
            ExcelUtils.exportExcel(list, null, "Sheet1", ReportNewCustomerRebuyBJTotal.class, "保洁新客首次履约后T+7复购率.xls", response);
            return null;
        }
    }

    @ApiOperation("报表数据更新时间")
    @GetMapping("etlTime")
    @LogRecordFiled(funName = "报表数据更新时间")
    public Object newCustomerTradeDetail(){
        return ResponseUtil.ok(spMapper.getEtlTime("select DATE_FORMAT(max(update_time),'%Y-%m-%d %H:%i') from trade_order"));
    }

    @ApiOperation("2022年双11活动")
    @GetMapping("double11")
    @LogRecordFiled(funName = "2022年双11活动销售数据")
    public Object double11(Integer type){
        return ResponseUtil.ok(reportService.getMarketActivity202211(type));
    }

}
