package com.jzlife.report.controller;

import java.io.IOException;
import java.util.List;

import com.jzlife.report.anno.LogRecordFiled;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;
import org.springframework.web.multipart.MultipartFile;

import com.alibaba.excel.EasyExcel;
import com.jzlife.report.dto.EtlGoalDto;
import com.jzlife.report.entity.hmorder.EtlGoal;
import com.jzlife.report.service.hmorder.SalesTargetValueService;
import com.jzlife.report.util.ExcelListener;
import com.jzlife.report.util.ResponseUtil;
import com.jzlife.report.util.page.PagedResult;

import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;

@Api(value = "销售目标值", description = "销售目标值")
@RequestMapping("salesTarget")
@RestController
public class SalesTargetValueController {

	@Autowired
	SalesTargetValueService salesTargetValueService;

	@ApiOperation("销售目标导入")
	@PostMapping("/import")
	@LogRecordFiled(funName = "销售目标导入")
	public Object uploadExcel(@RequestParam(value = "file") MultipartFile file, @RequestParam Long updateUser)
			throws IOException {

		ExcelListener<EtlGoal> listener = new ExcelListener<EtlGoal>();
		// ExcelModel 上面创建的实体类
		EasyExcel.read(file.getInputStream(), EtlGoal.class, listener).sheet(0).headRowNumber(1).doReadSync();

		List<EtlGoal> datas = listener.getDatas();
		return salesTargetValueService.insertTargetValue(datas,updateUser);

		// 导入如果存在条件相同的就更新，同时json保存到记录，不存在就新增

		// 提供单条修改，新增，删除，不校验目标值
		// 查询列表
	}

	@ApiOperation("销售目标单条修改")
	@PostMapping("/update")
	@LogRecordFiled(funName = "销售目标单条修改")
	public Object update(@RequestBody EtlGoal e) {
		salesTargetValueService.updateTargetValue(e);
		return ResponseUtil.ok();
	}

	@ApiOperation("销售目标单条新增")
	@PostMapping("/add")
	@LogRecordFiled(funName = "销售目标单条新增")
	public Object add(@RequestBody EtlGoal e) {
		salesTargetValueService.addTargetValue(e);
		return ResponseUtil.ok();
	}

	@ApiOperation("销售目标单条删除")
	@GetMapping("/delete")
	@LogRecordFiled(funName = "销售目标单条删除")
	public Object delete(Integer id) {
		salesTargetValueService.deleteTargetValue(id);
		return ResponseUtil.ok();
	}

	@ApiOperation("列表查询")
	@PostMapping("/pageData")
	@LogRecordFiled(funName = "列表查询")
	public Object pageData(@RequestBody EtlGoalDto dto) {
		PagedResult<EtlGoal> pageDataList = salesTargetValueService.pageDataList(dto);
		return ResponseUtil.ok(pageDataList);
	}

	@ApiOperation("字典数据")
	@GetMapping("/dictData")
	public Object dictData() {
		return ResponseUtil.ok();
	}
}
