package com.jzlife.report.controller;

import com.jzlife.report.anno.LogRecordFiled;
import com.jzlife.report.dto.*;
import com.jzlife.report.service.newreport.ServiceReportService;
import com.jzlife.report.util.ExcelUtils;
import com.jzlife.report.util.ResponseUtil;
import com.jzlife.report.util.page.BeanUtil;
import com.jzlife.report.util.page.PagedResult;
import com.jzlife.report.vo.DayReportBaseVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.RestController;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.util.List;

@Api(value = "服务力看板", description = "服务力薪资")
@RestController
@RequestMapping(value = "servicer")
public class ServiceController {

    @Autowired
    private ServiceReportService reportService;

    @ApiOperation("服务力薪资列表")
    @GetMapping("salaryList")
    @LogRecordFiled(funName = "服务力薪资列表")
    public Object salaryList(
            @RequestParam(required = false) String city,
            @RequestParam(required = false) String station,
            @RequestParam(required = false) String workerName,
            @RequestParam(required = true) String month,
            @RequestParam(defaultValue = "1") Integer pageIndex,
            @RequestParam(defaultValue = "50") Integer pageSize,
            @RequestParam(defaultValue = "false") Boolean isExport,
            HttpServletResponse response) throws Exception {
        if (StringUtils.isBlank(month)) {
            return ResponseUtil.fail("月份不能为空");
        }
        DayReportBaseVo vo = new DayReportBaseVo();
        vo.setCity(city);
        vo.setServiceStation(station);
        vo.setWorkerName(workerName);
        vo.setMonth(month.replace("-", ""));
        if(isExport){
            vo.setStartPage(1);
            vo.setLimit(10000);
        }
        else{
            vo.setStartPage(pageIndex);
            vo.setLimit(pageSize);
        }

        List<ReportServiceSalary> list = reportService.serviceSalaryList(vo);
        if (isExport == false) {
            Integer total = list.size() > 0 ? list.get(0).getTotal() : 0;
            PagedResult<ReportServiceSalary> pagedResult = BeanUtil.toPagedResult(list);
            pagedResult.setTotal(total);
            pagedResult.setPageNo(pageIndex);
            pagedResult.setPageSize(pageSize);
            pagedResult.setStartPage(1);
            pagedResult.setEndPage((total / pageSize) + 1);
            pagedResult.setPages(pagedResult.getEndPage());
            return ResponseUtil.ok(pagedResult);
        } else {
            ExcelUtils.exportExcel(list, null, "Sheet1", ReportServiceSalary.class, "服务力薪资列表.xls", response);
            return null;
        }
    }

    @ApiOperation("服务力服务单次保洁后，7日内的复购情况")
    @GetMapping("rebuy")
    @LogRecordFiled(funName = "服务力服务单次保洁后，7日内的复购情况")
    public Object rebuy(
            @RequestParam(required = true) String month,
            @RequestParam(required = false) String city,
            @RequestParam(defaultValue = "1") Integer pageIndex,
            @RequestParam(defaultValue = "50") Integer pageSize,
            @RequestParam(defaultValue = "false") Boolean isExport,
            HttpServletResponse response, HttpServletRequest request) throws Exception {

        if (StringUtils.isBlank(month)) {
            return ResponseUtil.fail("月份不能为空");
        }
        DayReportBaseVo vo = new DayReportBaseVo();
        vo.setCity(city);
        vo.setMonth(month);
        if(isExport){
            vo.setStartPage(1);
            vo.setLimit(10000);
        }
        else{
            vo.setStartPage(pageIndex);
            vo.setLimit(pageSize);
        }

        List<ReportServiceRebuy> list = reportService.serviceRebuyList(vo);
        if (isExport == false) {
            Integer total = list.size() > 0 ? list.get(0).getRowTotal() : 0;
            PagedResult<ReportServiceRebuy> pagedResult = BeanUtil.toPagedResult(list);
            pagedResult.setTotal(total);
            pagedResult.setPageNo(pageIndex);
            pagedResult.setPageSize(pageSize);
            pagedResult.setStartPage(1);
            pagedResult.setEndPage((total / pageSize) + 1);
            pagedResult.setPages(pagedResult.getEndPage());
            return ResponseUtil.ok(pagedResult);
        } else {
            ExcelUtils.exportExcel(list, null, "Sheet1", ReportServiceRebuy.class, "服务力复购列表.xls", response);
            return null;
        }
    }


    @ApiOperation("服务力服务单次保洁后，7日内的复购明细")
    @GetMapping("rebuyDetail")
    @LogRecordFiled(funName = "服务力服务单次保洁后，7日内的复购明细")
    public Object rebuyDetail(
            @RequestParam(required = true) String month,
            @RequestParam(required = false) String city,
            @RequestParam(defaultValue = "1") Integer pageIndex,
            @RequestParam(defaultValue = "50") Integer pageSize,
            @RequestParam(defaultValue = "false") Boolean isExport,
            HttpServletResponse response, HttpServletRequest request) throws Exception {
        if (StringUtils.isBlank(month)) {
            return ResponseUtil.fail("月份不能为空");
        }
        DayReportBaseVo vo = new DayReportBaseVo();
        vo.setCity(city);
        vo.setMonth(month);
        if(isExport){
            vo.setStartPage(1);
            vo.setLimit(40000);
        }
        else{
            vo.setStartPage(pageIndex);
            vo.setLimit(pageSize);
        }

        List<ReportServiceRebuy> list = reportService.serviceRebuyDetailList(vo);
        if (isExport == false) {
            Integer total = list.size() > 0 ? list.get(0).getRowTotal() : 0;
            PagedResult<ReportServiceRebuy> pagedResult = BeanUtil.toPagedResult(list);
            pagedResult.setTotal(total);
            pagedResult.setPageNo(pageIndex);
            pagedResult.setPageSize(pageSize);
            pagedResult.setStartPage(1);
            pagedResult.setEndPage((total / pageSize) + 1);
            pagedResult.setPages(pagedResult.getEndPage());
            return ResponseUtil.ok(pagedResult);
        } else {
            ExcelUtils.exportExcel(list, null, "Sheet1", ReportServiceRebuy.class, "服务力复购列表.xls", response);
            return null;
        }
    }


    @ApiOperation("服务力服务品类后，7日内的复购明细")
    @GetMapping("rebuyPLDetail")
    @LogRecordFiled(funName = "服务力服务品类后，7日内的复购明细")
    public Object rebuyPlDetail(
            @RequestParam(required = true) String month,
            @RequestParam(required = false) String city,
            @RequestParam(defaultValue = "1") Integer pageIndex,
            @RequestParam(defaultValue = "50") Integer pageSize,
            @RequestParam(defaultValue = "false") Boolean isExport,
            HttpServletResponse response, HttpServletRequest request) throws Exception {
        if (StringUtils.isBlank(month)) {
            return ResponseUtil.fail("月份不能为空");
        }
        DayReportBaseVo vo = new DayReportBaseVo();
        vo.setCity(city);
        vo.setMonth(month);
        if(isExport){
            vo.setStartPage(1);
            vo.setLimit(40000);
        }
        else{
            vo.setStartPage(pageIndex);
            vo.setLimit(pageSize);
        }

        List<ReportServiceRebuy> list = reportService.serviceRebuyPLDetailList(vo);
        if (isExport == false) {
            Integer total = list.size() > 0 ? list.get(0).getRowTotal() : 0;
            PagedResult<ReportServiceRebuy> pagedResult = BeanUtil.toPagedResult(list);
            pagedResult.setTotal(total);
            pagedResult.setPageNo(pageIndex);
            pagedResult.setPageSize(pageSize);
            pagedResult.setStartPage(1);
            pagedResult.setEndPage((total / pageSize) + 1);
            pagedResult.setPages(pagedResult.getEndPage());
            return ResponseUtil.ok(pagedResult);
        } else {
            ExcelUtils.exportExcel(list, null, "Sheet1", ReportServiceRebuy.class, "服务力复购列表.xls", response);
            return null;
        }
    }

    @ApiOperation("老客续约率 （周期卡内最后一次服务完成后的30天内有未使用的、未履约的、未预约的、服务中的订单）")
    @GetMapping("rebuyOldCustomerDetail")
    @LogRecordFiled(funName = "周期卡最后一次完成后T+30复购")
    public Object rebuyOldCustomerDetail(
            @RequestParam(required = true) String month,
            @RequestParam(required = false) String city,
            @RequestParam(defaultValue = "1") Integer pageIndex,
            @RequestParam(defaultValue = "50") Integer pageSize,
            @RequestParam(defaultValue = "false") Boolean isExport,
            HttpServletResponse response, HttpServletRequest request) throws Exception {
        if (StringUtils.isBlank(month)) {
            return ResponseUtil.fail("月份不能为空");
        }
        DayReportBaseVo vo = new DayReportBaseVo();
        vo.setCity(city);
        vo.setMonth(month);
        if(isExport){
            vo.setStartPage(1);
            vo.setLimit(40000);
        }
        else{
            vo.setStartPage(pageIndex);
            vo.setLimit(pageSize);
        }

        List<ReportOldCustomerRebuyDetail> list = reportService.serviceOldCustomerRebuyDetailList(vo);
        if (isExport == false) {
            Integer total = list.size() > 0 ? list.get(0).getRowTotal() : 0;
            PagedResult<ReportOldCustomerRebuyDetail> pagedResult = BeanUtil.toPagedResult(list);
            pagedResult.setTotal(total);
            pagedResult.setPageNo(pageIndex);
            pagedResult.setPageSize(pageSize);
            pagedResult.setStartPage(1);
            pagedResult.setEndPage((total / pageSize) + 1);
            pagedResult.setPages(pagedResult.getEndPage());
            return ResponseUtil.ok(pagedResult);
        } else {
            ExcelUtils.exportExcel(list, null, "Sheet1", ReportOldCustomerRebuyDetail.class, "服务力复购列表.xls", response);
            return null;
        }
    }


    @ApiOperation("散单服务单明细（包括保洁、品类)")
    @GetMapping("newCustomerTradeDetail")
    @LogRecordFiled(funName = "散单新客 首单服务明细")
    public Object newCustomerTradeDetail(
            @RequestParam(required = true) String startTime,
            @RequestParam(required = false) String city,
            @RequestParam(required = false) String type,//保洁、品类
            @RequestParam(defaultValue = "1") Integer pageIndex,
            @RequestParam(defaultValue = "50") Integer pageSize,
            @RequestParam(defaultValue = "false") Boolean isExport,
            HttpServletResponse response, HttpServletRequest request) throws Exception {
        if (StringUtils.isBlank(startTime)||StringUtils.isBlank(city)) {
            return ResponseUtil.fail("服务日期或城市不能为空");
        }
        if(StringUtils.isBlank(type)){
            return ResponseUtil.fail("业务类型不能为空");
        }
        DayReportBaseVo vo = new DayReportBaseVo();
        vo.setCity(city);
        vo.setStartTime(startTime);
        if(isExport){
            vo.setStartPage(1);
            vo.setLimit(40000);
        }
        else{
            vo.setStartPage(pageIndex);
            vo.setLimit(pageSize);
        }

        List<ReportNewCustomerTradeDetail> list = reportService.serviceNewCustomerTradeDetail(vo,type);
        if (isExport == false) {
            Integer total = list.size() > 0 ? list.get(0).getRowTotal() : 0;
            PagedResult<ReportNewCustomerTradeDetail> pagedResult = BeanUtil.toPagedResult(list);
            pagedResult.setTotal(total);
            pagedResult.setPageNo(pageIndex);
            pagedResult.setPageSize(pageSize);
            pagedResult.setStartPage(1);
            pagedResult.setEndPage((total / pageSize) + 1);
            pagedResult.setPages(pagedResult.getEndPage());
            return ResponseUtil.ok(pagedResult);
        } else {
            ExcelUtils.exportExcel(list, null, "Sheet1", ReportNewCustomerTradeDetail.class, "散单新客首单服务明细.xls", response);
            return null;
        }
    }

    @ApiOperation("周期服务单明细")
    @GetMapping("cycleTradeDetail")
    @LogRecordFiled(funName = "周期服务单明细")
    public Object cycleTradeDetail(
            @RequestParam(required = true) String startTime,
            @RequestParam(required = false) String city,
            @RequestParam(defaultValue = "1") Integer pageIndex,
            @RequestParam(defaultValue = "50") Integer pageSize,
            @RequestParam(defaultValue = "false") Boolean isExport,
            HttpServletResponse response, HttpServletRequest request) throws Exception {
        if (StringUtils.isBlank(startTime) || StringUtils.isBlank(city)) {
            return ResponseUtil.fail("服务日期或城市不能为空");
        }
        DayReportBaseVo vo = new DayReportBaseVo();
        vo.setCity(city);
        vo.setStartTime(startTime);
        if (isExport) {
            vo.setStartPage(1);
            vo.setLimit(40000);
        } else {
            vo.setStartPage(pageIndex);
            vo.setLimit(pageSize);
        }

        if (isExport == false) {
            return ResponseUtil.ok(reportService.serviceCycleTradeDetail(vo, isExport));
        } else {
            List<ReportCycleTradeDetail> list = (List<ReportCycleTradeDetail>) reportService.serviceCycleTradeDetail(vo, isExport);
            ExcelUtils.exportExcel(list, null, "Sheet1", ReportCycleTradeDetail.class, "周期单服务明细.xls", response);
            return null;
        }
    }

    @ApiOperation("报表数据更新时间")
    @GetMapping("etlTime")
    @LogRecordFiled(funName = "报表数据更新时间")
    public Object newCustomerTradeDetail(){
        return ResponseUtil.ok(reportService.getEtlTime());
    }

}
