package com.jzlife.report.controller;


import com.github.pagehelper.PageInfo;
import com.jzlife.report.anno.LogRecordFiled;
import com.jzlife.report.dao.manager.CrmTicketConsultTypeMapper;
import com.jzlife.report.dto.CrmTicketConsultType;
import com.jzlife.report.dto.export.OnceFilingTicketExportDto;
import com.jzlife.report.dto.export.OnlineChatSummaryExport;
import com.jzlife.report.dto.export.OnlineChathzExport;
import com.jzlife.report.dto.export.ServantSignOrderExport;
import com.jzlife.report.excelExport.DayCustomerClaimDetailDTO;
import com.jzlife.report.excelExport.ReportComplaintmanagementDTO;
import com.jzlife.report.exception.BusinessException;
import com.jzlife.report.service.hmorder.StationSerevice;
import com.jzlife.report.service.newreport.ServiceQualityReportService;
import com.jzlife.report.util.ExcelUtils;
import com.jzlife.report.util.ResponseUtil;
import com.jzlife.report.vo.DayReportBaseVo;
import com.jzlife.report.vo.ReportBaseVo;
import com.jzlife.report.vo.ReportCustomerClaimAndPowerVo;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.*;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import java.time.LocalDate;
import java.time.format.DateTimeFormatter;
import java.time.temporal.ChronoUnit;
import java.time.temporal.TemporalAdjusters;
import java.util.List;
import java.util.Map;

@Api(value = "服务质量看板", description = "服务质量看板")
@RestController
@RequestMapping(value = "report")
public class ServiceQualityReportController {

    @Autowired
    private ServiceQualityReportService service;

    @Autowired
    StationSerevice stationSerevice;

    @Autowired
    private CrmTicketConsultTypeMapper crmTicketConsultTypeMapper;


    @ApiOperation(value = "服务质量情况堆叠折线图", httpMethod = "POST", notes = "时间格式为：yyyy-MM")
    @PostMapping("serviceQualityStackLine")
    @LogRecordFiled(funName = "服务质量情况堆叠折线图")
    public Object serviceQualityStackLine(@Validated @RequestBody ReportBaseVo vo, HttpServletRequest request) {
        String errorMsg = checkChooseTime(vo.getStartTime(), vo.getEndTime());
        if (StringUtils.isNotBlank(errorMsg)) {
            return ResponseUtil.fail400(errorMsg);
        }
        if (StringUtils.isEmpty(vo.getCity())) {
            String citys = request.getHeader("allowCity");
            if (StringUtils.isNotEmpty(citys)) {
                vo.setCity(citys);
            }
        }
        Map<String, Object> data = service.serviceQualityStackLine(vo);
        return ResponseUtil.ok(data);
    }


    @ApiOperation(value = "客诉数和服务能力折线图", httpMethod = "POST", notes = "时间可选，在颗粒度为月份时格式为yyyy-MM，在颗粒度为天时 为 yyyy-MM-dd ")
    @PostMapping("customerClaimAndServicePowerLineGraph")
    @LogRecordFiled(funName = "客诉数和服务能力折线图")
    public Object customerClaimAndServicePowerLineGraph(@Validated @RequestBody ReportCustomerClaimAndPowerVo vo, HttpServletRequest request) {
        if (vo.getTimeType() == 1 && (StringUtils.isEmpty(vo.getStartMonth()) || StringUtils.isEmpty(vo.getEndMonth()))) {
            return ResponseUtil.fail400("开始结束月份不可为空");
        }
        if (vo.getTimeType() == 2 && (StringUtils.isEmpty(vo.getStartTime()) || StringUtils.isEmpty(vo.getEndTime()))) {
            return ResponseUtil.fail400("开始结束日期不可为空");
        }

        if (StringUtils.isEmpty(vo.getCity())) {
            String citys = request.getHeader("allowCity");
            if (StringUtils.isNotEmpty(citys)) {
                vo.setCity(citys);
            }
        }
        Map<String, Object> data = service.customerClaimAndServicePowerLineGraph(vo);
        return ResponseUtil.ok(data);
    }


    private String checkChooseTime(String startTime, String endTime) {
        LocalDate start = LocalDate.parse(startTime + "-01", DateTimeFormatter.ofPattern("yyyy-MM-dd"));
        LocalDate end = LocalDate.parse(endTime + "-01", DateTimeFormatter.ofPattern("yyyy-MM-dd")).with(TemporalAdjusters.lastDayOfMonth());
        if (start.isAfter(end) || start.isEqual(end)) {
            return "开始时间不可大于或等于结束时间！";
        }
        if (end.getYear() - start.getYear() > 1) {
            return "所选时间跨度不可超过1年";
        }
        if (start.getYear() + 2 < LocalDate.now().getYear()) {
            return "仅支持统计最近两年的数据";
        }
        return "";
    }

    @ApiOperation(value = "客诉一级指标", httpMethod = "GET", notes = "包含进线接听率、外呼接听率、现存客诉单、平均处理时效、S级工单、>48小时工单")
    @GetMapping("claimSummaryAndPie")
    @LogRecordFiled(funName = "客诉一级指标")
    public Object claimSummaryAndPie() {
        Map<String, Object> data = service.claimSummaryAndPie();
        return ResponseUtil.ok(data);
    }


    @ApiOperation(value = "客诉明细", httpMethod = "POST", notes = "时间格式为：yyyy-MM-dd,查询选定时间段内的每日客诉明细 使用盒马服务站")
    @PostMapping("dayCustomerClaimDetail")
    @LogRecordFiled(funName = "客诉明细")
    public Object dayCustomerClaimDetail(@Validated @RequestBody DayReportBaseVo vo, HttpServletRequest request) {
//        if (StringUtils.isEmpty(vo.getCity())) {
//            String citys = request.getHeader("allowCity");
//            if (StringUtils.isNotEmpty(citys)) {
//                vo.setCity(citys);
//            }
//        }
        PageInfo<Map<String, Object>> data = service.dayCustomerClaimDetail(vo);
        return ResponseUtil.ok(data);
    }

    @ApiOperation(value = "客诉明细导出", httpMethod = "GET", notes = "时间格式为：yyyy-MM-dd,查询选定时间段内的每日客诉明细 使用盒马服务站,仅导出最新1万条")
    @GetMapping("dayCustomerClaimDetailExport")
    @LogRecordFiled(funName = "客诉明细导出")
    public void dayCustomerClaimDetailExport(@RequestParam(value = "city", required = false) String city,
                                             @RequestParam(value = "serviceStation", required = false) String serviceStation,
                                             @RequestParam(value = "serverName", required = false) String serverName,
                                             @RequestParam(value = "claimType", required = false) Integer claimType,
                                             @RequestParam(value = "consultValue", required = false) Integer[] consultValue,
                                             @RequestParam(value = "customerClaimLevel", required = false) Integer customerClaimLevel,
                                             @RequestParam(value = "ticketSn", required = false) String ticketSn,
                                             @RequestParam(value = "claimStatus", required = false) Integer claimStatus,
                                             @RequestParam(value = "startTime") String startTime,
                                             @RequestParam(value = "endTime") String endTime,
                                             String bookType,
                                             HttpServletRequest request,
                                             HttpServletResponse response
    ) throws Exception {
//        if (StringUtils.isEmpty(city)) {
//            String citys = request.getHeader("allowCity");
//            if (StringUtils.isNotEmpty(citys)) {
//                city = citys;
//            }
//        }
        if (StringUtils.isEmpty(startTime) || StringUtils.isEmpty(endTime)) {
            throw new BusinessException("开始时间或结束时间不可为空");
        }
        LocalDate start = LocalDate.parse(startTime, DateTimeFormatter.ofPattern("yyyy-MM-dd"));
        LocalDate end = LocalDate.parse(endTime, DateTimeFormatter.ofPattern("yyyy-MM-dd"));
        if (start.until(end, ChronoUnit.DAYS) > 31) {
            throw new BusinessException("所选时间跨度不可超过31天");
        }

        DayReportBaseVo baseVo = new DayReportBaseVo();
        baseVo.setCity(city);
        baseVo.setServiceStation(serviceStation);
        baseVo.setServerName(serverName);
        baseVo.setClaimType(claimType);
        baseVo.setConsultValue(consultValue);
        baseVo.setCustomerClaimLevel(customerClaimLevel);
        baseVo.setTicketSn(ticketSn);
        baseVo.setClaimStatus(claimStatus);
        baseVo.setStartTime(startTime);
        baseVo.setEndTime(endTime);
        List<DayCustomerClaimDetailDTO> list = service.dayCustomerClaimDetailNoPage(baseVo);
        if (list.size() > 0) {
            ExcelUtils.exportExcel(list, "服务质量-客诉明细", "客诉明细", DayCustomerClaimDetailDTO.class, "客诉明细", response);
        }

    }


    @ApiOperation(value = "获取工单咨询类型", httpMethod = "GET", notes = "获取维度表工单咨询类型(1,2两级)")
    @GetMapping("getTicketConsultType")
    @LogRecordFiled(funName = "获取工单咨询类型")
    public Object getTicketConsultType() {
        List<CrmTicketConsultType> roots = crmTicketConsultTypeMapper.selectByPid(0);
        for (CrmTicketConsultType type : roots) {
            type.setChild(crmTicketConsultTypeMapper.selectByPid(type.getId()));
        }
        return ResponseUtil.ok(roots);
    }


    @ApiOperation(value = "在线客服报表-明细")
    @GetMapping("onlineChatSummary")
    @LogRecordFiled(funName = "在线客服报表-明细")
    public Object onlineChatSummary(@RequestParam(defaultValue = "1") Integer pageNo,
                                    @RequestParam(defaultValue = "50") Integer pageSize, String startDate,
                                    String endDate, String customerId, String userName,
                                    @RequestParam(defaultValue = "false") Boolean isExport,
                                    HttpServletRequest request, HttpServletResponse response) throws Exception {

        startDate = startDate + " 00:00:00";
        endDate = endDate + " 23:59:59";

        Object result = service.onlineChatSummary(pageNo, pageSize, startDate, endDate, customerId, userName, isExport);
        if (isExport) {
            List<OnlineChatSummaryExport> list = (List<OnlineChatSummaryExport>) result;
            if (list != null && list.size() > 0) {
                ExcelUtils.exportExcel(list, null, "Sheet1", OnlineChatSummaryExport.class, "在线客服汇总.xls", response);
            }
        }

        return ResponseUtil.ok(result);
    }

    @ApiOperation(value = "在线客服报表-汇总")
    @GetMapping("onlineChathz")
    @LogRecordFiled(funName = "在线客服报表-汇总")
    public Object onlineChathz(@RequestParam(defaultValue = "1") Integer pageNo,
                               @RequestParam(defaultValue = "50") Integer pageSize, String startDate,
                               String endDate, String userName,
                               @RequestParam(defaultValue = "false") Boolean isExport,
                               HttpServletRequest request, HttpServletResponse response) throws Exception {
        Object data = service.onlineChathz(pageNo, pageSize, startDate, endDate, userName, isExport);
        if (isExport) {
            List<OnlineChathzExport> list = (List<OnlineChathzExport>) data;
            if (list != null && list.size() > 0) {
                ExcelUtils.exportExcel(list, null, "Sheet1", OnlineChathzExport.class, "在线客服汇总.xls", response);
            }
        }
        return ResponseUtil.ok(data);
    }


    // 投诉治理
    @ApiOperation(value = "投诉治理明细导出", httpMethod = "GET", notes = "时间格式为：yyyy-MM-dd,查询选定时间段内的每日客诉明细 使用盒马服务站")
    @GetMapping("complaintManagementExport")
    @LogRecordFiled(funName = "投诉治理明细导出")
    public void complaintManagementExport(@RequestParam(value = "city", required = false) String city,
                                          @RequestParam(value = "serviceStation", required = false) String serviceStation,
                                          @RequestParam(value = "serverName", required = false) String serverName,
                                          @RequestParam(value = "ticketSn", required = false) String ticketSn,
                                          @RequestParam(value = "hemaWorkOrder", required = false) String hemaWorkOrder,
                                          @RequestParam(value = "startTime") String startTime,
                                          @RequestParam(value = "endTime") String endTime,
                                          HttpServletRequest request,
                                          HttpServletResponse response
    ) throws Exception {
        if (StringUtils.isEmpty(city)) {
            String citys = request.getHeader("allowCity");
            if (StringUtils.isNotEmpty(citys)) {
                city = citys;
            }
        }
        List<ReportComplaintmanagementDTO> list = service.complaintManagementExport(city, serviceStation, serverName, ticketSn, hemaWorkOrder, startTime, endTime);
        if (list.size() > 0) {
            ExcelUtils.exportExcel(list, "服务质量-投诉治理", "投诉治理", ReportComplaintmanagementDTO.class, "投诉治理", response);
        }

    }

    @ApiOperation(value = "投诉治理明细", httpMethod = "GET", notes = "时间格式为：yyyy-MM-dd,查询选定时间段内的每日客诉明细 使用盒马服务站")
    @GetMapping("complaintManagement")
    @LogRecordFiled(funName = "投诉治理明细")
    public Object complaintManagement(@RequestParam(value = "city", required = false) String city,
                                      @RequestParam(value = "serviceStation", required = false) String serviceStation,
                                      @RequestParam(value = "serverName", required = false) String serverName,
                                      @RequestParam(value = "ticketSn", required = false) String ticketSn,
                                      @RequestParam(value = "hemaWorkOrder", required = false) String hemaWorkOrder,
                                      @RequestParam(value = "startTime") String startTime,
                                      @RequestParam(value = "endTime") String endTime,
                                      HttpServletRequest request,
                                      @RequestParam(required = false, defaultValue = "1") String pageNo,
                                      @RequestParam(required = false, defaultValue = "50") String pageSize
    ) {
        if (StringUtils.isEmpty(city)) {
            String citys = request.getHeader("allowCity");
            if (StringUtils.isNotEmpty(citys)) {
                city = citys;
            }
        }
        PageInfo<ReportComplaintmanagementDTO> info = service.complaintManagement(city, serviceStation, serverName,
                ticketSn, hemaWorkOrder, startTime, endTime, pageNo, pageSize);
        return ResponseUtil.ok(info);
    }

    @ApiOperation(value = "一键报备")
    @GetMapping("onceFilingTicket")
    @LogRecordFiled(funName = "一键报备")
    public Object onceFilingTicket(String city, String startDate, String endDate,
                                   @RequestParam(defaultValue = "1") Integer pageNo,
                                   @RequestParam(defaultValue = "50") Integer pageSize,
                                   @RequestParam(defaultValue = "false") Boolean isExport,
                                   HttpServletRequest request, HttpServletResponse response) throws Exception {
        if (StringUtils.isEmpty(city)) {
            String citys = request.getHeader("allowCity");
            if (StringUtils.isNotEmpty(citys)) {
                city = citys;
            }
        }
        // call sp_report_once_filing('2022-06-01','2022-06-30');
        Object res = service.onceFilingTicket(pageNo, pageSize, startDate, endDate, city, isExport);
        if (isExport) {
            List<OnceFilingTicketExportDto> list = (List<OnceFilingTicketExportDto>) res;
            if (list != null && list.size() > 0) {
                ExcelUtils.exportExcel(list, null, "Sheet1", OnceFilingTicketExportDto.class, "一键报备.xls", response);
            }
        }

        return res;
    }
}
