package com.jzlife.report.schedule;

import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.enums.CellDataTypeEnum;
import com.alibaba.excel.exception.ExcelDataConvertException;
import com.alibaba.excel.metadata.data.CellData;
import com.alibaba.excel.read.builder.ExcelReaderBuilder;
import com.alibaba.excel.read.builder.ExcelReaderSheetBuilder;
import com.jzlife.report.constant.ImportExcelConstant;
import com.jzlife.report.dao.nurse.Report30daysProductStocksMapper;
import com.jzlife.report.dao.reports.HmScheduleMapper;
import com.jzlife.report.entity.reports.ImportDataConfig;
import com.jzlife.report.entity.reports.ImportDataRecord;
import com.jzlife.report.service.*;
import com.jzlife.report.service.hmorder.ImportExcelService;
import com.jzlife.report.service.newreport.ServicePowerReportService;
import com.jzlife.report.service.report.FileScannerReadExcelService;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.io.FileUtils;
import org.apache.poi.xssf.usermodel.XSSFWorkbook;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Value;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import java.io.*;
import java.nio.file.Files;
import java.time.LocalDate;
import java.time.format.DateTimeFormatter;
import java.util.List;
import java.util.Map;

/**
 * 大数据量导入，读取磁盘目录文件
 *
 * @author lurz
 */

@Slf4j
@Component
public class ImportExcelTask {

    @Autowired
    FileScannerReadExcelService fileScanner;

    @Autowired
    ServicePowerReportService servicePowerReportService;

    @Autowired
    ImportExcelService importExcelService;


    @Value("${spring.profiles.active}")
    private String env;

    @Autowired
    private Report30daysProductStocksMapper daysStocksMapper;

    @Autowired
    private HmScheduleMapper scheduleMapper;

    // 1、数据库加入配置类型
    // 2、实现业务插入方法
    // 3、当前runImportTask方法里加入判断p.getTypeCode().equals(新的导入类型)
    // 4、在EasyExcelListener中的doAfterAllAnalysed 和invoke加入调用 逻辑

    //    @Scheduled(cron = "0 0/10 9-22 * * ?") // 每天早上9点到晚上10点,10分钟跑一次
    public void runImportTask() {
        if (env.equals("pro")) {
            // 读取前准备
            List<ImportDataRecord> importList = fileScanner.readFileAndReName();
            log.info("预备读取excel文件数：" + importList.size());
            // 开始读取导入
            for (ImportDataRecord p : importList) {
                if (p.getTypeCode().equals(ImportExcelConstant.TYPE_CODE_1002)) {
                    servicePowerReportService.import30DaysProductStockTask(p.getFilePath(), p);
                }
            }
            log.info("开始删除历史15天前文件");
            // 删除历史15天前文件
            fileScanner.deleteFile();
            // 扫描新文件
            List<ImportDataRecord> newFileList = fileScanner.fileScanner();
            log.info("扫描新的excel文件数：" + newFileList.size());
        }

    }


    @SneakyThrows({RuntimeException.class, IOException.class, ClassNotFoundException.class})
    @Scheduled(cron = "0 20 19 * * ?") // 每天早上10点20分跑一次
    public void importReportData() {
        if (env.equals("pro")) {
            List<ImportDataConfig> configList = fileScanner.getDayScanProfiles();
            String monthDay = LocalDate.now().format(DateTimeFormatter.ofPattern("yyyyMMdd")).substring(4);
            for (ImportDataConfig importDataConfig : configList) {
                String realFilePath = importDataConfig.getScanPath().replace("@day", monthDay);
                List<File> files = fileScanner.preProcessImport(realFilePath, importDataConfig);
                if (files != null && !files.isEmpty()) {
                    for (File file : files) {
                        if (!file.getName().contains("30天商品库存")) {
                            File realFile = new File(realFilePath, file.getName());
                            InputStream inputStream = Files.newInputStream(realFile.toPath());
                            XSSFWorkbook workbook = new XSSFWorkbook(inputStream);
                            int numberOfSheets = workbook.getNumberOfSheets();
                            inputStream.close();
                            if (importDataConfig.getTypeCode().equals(ImportExcelConstant.TYPE_CODE_1003)) {
                                processStationStock(importDataConfig, file, numberOfSheets);
                            }
                            if (importDataConfig.getTypeCode().equals(ImportExcelConstant.TYPE_CODE_1004)) {
                                processServerSignInfo(importDataConfig, file, numberOfSheets);
                            }
                            if (importDataConfig.getTypeCode().equals(ImportExcelConstant.TYPE_CODE_1005)) {
                                //处理评价数据
                                processEvaluation(importDataConfig, file, numberOfSheets);
                            }
                            if (importDataConfig.getTypeCode().equals(ImportExcelConstant.TYPE_CODE_1006)) {
                                //处理请假数据
                                processAskForLeave(importDataConfig, file, numberOfSheets);
                            }
                            if (importDataConfig.getTypeCode().equals(ImportExcelConstant.TYPE_CODE_1007)) {
                                //处理排班数据
                                processSchedule(importDataConfig, file, numberOfSheets);
                            }
                            if (importDataConfig.getTypeCode().equals(ImportExcelConstant.TYPE_CODE_1009)) {
                                //处理排班请假数据
                                processScheduleAndAskLeave(importDataConfig, file);
                            }
                            if (importDataConfig.getTypeCode().equals(ImportExcelConstant.TYPE_CODE_1010)) {
                                //处理60天服务站技师库存
                                processStationPersonStock(importDataConfig, file, numberOfSheets);
                            }
                        } else if (importDataConfig.getTypeCode().equals(ImportExcelConstant.TYPE_CODE_1008)) {
                            //处理30天商品库存数据
                            if (file.getName().endsWith(".csv")) {
                                setUTF8(file);
                            }
                            daysStocksMapper.trancatTable();
                            processLifeService(importDataConfig, file, daysStocksMapper);
                        }
                    }

                }
            }

            scheduleMapper.clearRestRepeatSchedule();

            String maxBatchNo = scheduleMapper.getNewBatchNo();
            Map<String, String> maxMinDate = scheduleMapper.getMaxMinDate(maxBatchNo);
            if (maxMinDate != null) {
                scheduleMapper.insert_worker_schedule_v2(maxMinDate.get("maxdate"), maxMinDate.get("mindate"));
                log.info("开始执行insert_worker_schedule_v2");
            }

        }

    }

    private void processStationPersonStock(ImportDataConfig importDataConfig, File file, int numberOfSheets) throws ClassNotFoundException {
        int id = servicePowerReportService.get14DayPersonBatchId();
        ExcelReaderBuilder read = EasyExcel.read(file).head(Class.forName(importDataConfig.getClasspath()));
        read.registerReadListener(new Hm60StationPersonListener(id));
        ExcelReaderSheetBuilder sheetBuilder = read.sheet(numberOfSheets - 1);
        sheetBuilder.doRead();
        servicePowerReportService.callProcedure();
    }

    private void setUTF8(File file) throws IOException {
        FileInputStream in = new FileInputStream(file);
        byte[] b = new byte[3];
        int read = in.read(b);
        in.close();
        //判断文件bom
        if (b[0] == -17 && b[1] == -69 && b[2] == -65) {
            log.info("current file is utf-8");
        } else {
            log.info("current file is not utf-8, convert to utf-8");
            String content = FileUtils.readFileToString(file, "GB2312");
            FileOutputStream outputStream = new FileOutputStream(file);
            outputStream.write(new byte[]{(byte) 0xEF, (byte) 0xBB, (byte) 0xBF});
            FileInputStream x = new FileInputStream(file);
            byte[] byt = new byte[1024];
            if (x.read(byt) != -1) {
                outputStream.write(content.getBytes());
            }
            outputStream.flush();
            x.close();
            outputStream.close();
        }
    }

    private void processScheduleAndAskLeave(ImportDataConfig importDataConfig, File file) throws ClassNotFoundException {
        importDataConfig.setClasspath("com.jzlife.report.dto.excelimport.HmScheduleOnleaveExcel");
        processAskForLeave(importDataConfig, file, 2);
        importDataConfig.setClasspath("com.jzlife.report.dto.excelimport.HmScheduleExcel");
        processSchedule(importDataConfig, file, 1);

    }

    private void processLifeService(ImportDataConfig importDataConfig, File file, Report30daysProductStocksMapper daysStocksMapper) throws ClassNotFoundException {
        EasyExcel.read(file).head(Class.forName(importDataConfig.getClasspath())).registerReadListener(new LifeServiceListener(daysStocksMapper)).sheet(0).doRead();
    }

    private void processSchedule(ImportDataConfig importDataConfig, File file, int numberOfSheets) throws ClassNotFoundException {
        ExcelReaderBuilder head = EasyExcel.read(file).head(Class.forName(importDataConfig.getClasspath()));
        String times = System.currentTimeMillis() + "";
        head.registerReadListener(new HmScheduleListener(times));
        ExcelReaderSheetBuilder builder = head.sheet(numberOfSheets - 1);
        builder.doRead();
    }

    private void processAskForLeave(ImportDataConfig importDataConfig, File file, int numberOfSheets) throws ClassNotFoundException {
        ExcelReaderBuilder head = EasyExcel.read(file).head(Class.forName(importDataConfig.getClasspath()));
        String times = System.currentTimeMillis() + "";
        head.registerReadListener(new LeaveListener(times));
        ExcelReaderSheetBuilder builder = head.sheet(numberOfSheets - 1);
        builder.doRead();
    }

    private void processEvaluation(ImportDataConfig importDataConfig, File file, int numberOfSheets) throws ClassNotFoundException {
        ExcelReaderBuilder head = EasyExcel.read(file).head(Class.forName(importDataConfig.getClasspath()));
        String times = System.currentTimeMillis() + "";
        head.registerReadListener(new EvaluationListeners(times));
        ExcelReaderSheetBuilder builder = head.sheet(numberOfSheets - 1);
        try {
            builder.doRead();
        } catch (RuntimeException e) {
            if (e instanceof ExcelDataConvertException) {
                ExcelDataConvertException excelDataConvertException = (ExcelDataConvertException) e;
                String cellMsg = "";
                CellData cellData = excelDataConvertException.getCellData();
                CellDataTypeEnum type = cellData.getType();
                if (type.equals(CellDataTypeEnum.NUMBER)) {
                    cellMsg = cellData.getNumberValue().toString();
                } else if (type.equals(CellDataTypeEnum.STRING)) {
                    cellMsg = cellData.getStringValue();
                } else if (type.equals(CellDataTypeEnum.BOOLEAN)) {
                    cellMsg = cellData.getBooleanValue().toString();
                }
                String errorMsg = String.format("excel表格:第%s行,第%s列,数据值为:%s,该数据值不符合要求,请检验后重新导入!<span style=\"color:red\">请检查其他的记录是否有同类型的错误!</span>", excelDataConvertException.getRowIndex() + 1, excelDataConvertException.getColumnIndex(), cellMsg);
                log.error(errorMsg);
            }
        }
    }

    private void processServerSignInfo(ImportDataConfig importDataConfig, File file, int numberOfSheets) throws ClassNotFoundException {
        ExcelReaderBuilder head = EasyExcel.read(file).head(Class.forName(importDataConfig.getClasspath()));
        String times = String.valueOf(System.currentTimeMillis());
        head.registerReadListener(new signInListener(times));
        ExcelReaderSheetBuilder builder = head.sheet(numberOfSheets - 1);
        builder.doRead();
    }

    //服务站库存
    private void processStationStock(ImportDataConfig importDataConfig, File file, int numberOfSheets) throws ClassNotFoundException {
        int id = servicePowerReportService.get14DayBatchId();
        ExcelReaderBuilder read = EasyExcel.read(file).head(Class.forName(importDataConfig.getClasspath()));
        read.registerReadListener(new ImportExcelListener(id));
        ExcelReaderSheetBuilder sheetBuilder = read.sheet(numberOfSheets - 1);
        sheetBuilder.doRead();
    }

}
