package com.jzlife.report.service;

import com.alibaba.excel.context.AnalysisContext;
import com.alibaba.excel.read.listener.ReadListener;
import com.jzlife.report.dao.reports.Report14daysCityStackRadioPersonMapper;
import com.jzlife.report.entity.reports.Report14daysCityStackRadioPerson;
import com.jzlife.report.util.SpringContextUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.BeanUtils;

import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.Date;

@Slf4j
public class Hm60StationPersonListener implements ReadListener<Report14daysCityStackRadioPerson> {

    private final int batchId;

    private final Report14daysCityStackRadioPersonMapper radioMapper;

    private final ArrayList<Report14daysCityStackRadioPerson> tempDatas = new ArrayList<>();

    public Hm60StationPersonListener(int batchId) {
        this.batchId = batchId;
        this.radioMapper = SpringContextUtils.getBeanByClass(Report14daysCityStackRadioPersonMapper.class);
    }

    @Override
    public void invoke(Report14daysCityStackRadioPerson data, AnalysisContext context) {
        if (data.getDate() == null) {
            return;
        }
        data.setBatchId(batchId);
        data.setImportDate(new Date());
        Report14daysCityStackRadioPerson dbData = new Report14daysCityStackRadioPerson();
        BeanUtils.copyProperties(data, dbData);
        dbData.setCity(data.getCity().substring(0, 2));
        try {
            dbData.setCurrentDayLeisureSourceCount(dbData.getCurrentDaySourceCount() - dbData.getCurrentDayUsedSourceCount());
        } catch (NumberFormatException e) {
            log.info("currentDayServerCount is not a number");
            log.info("current data is:{}", data);
        }
        tempDatas.add(dbData);
        int max_temp_count = 1000;
        if (tempDatas.size() >= max_temp_count) {
            radioMapper.batchInsert(tempDatas);
            tempDatas.clear();
        }

    }

    @Override
    public void doAfterAllAnalysed(AnalysisContext context) {
        log.info("read success");
        if (!tempDatas.isEmpty()) {
            radioMapper.batchInsert(tempDatas);
            tempDatas.clear();
        }
        radioMapper.filterDelete(batchId);
    }
}
