package com.jzlife.report.service.hmorder.impl;

import com.google.common.collect.Lists;
import com.jzlife.report.dao.hmorder.DiMerchandiseDisplayMapper;
import com.jzlife.report.dao.hmorder.DiRServicegmvLeftMapper;
import com.jzlife.report.dao.hmorder.DiRServicegmvRightMapper;
import com.jzlife.report.dao.hmorder.SalesReportMapper;
import com.jzlife.report.dao.nurse.SysCityMapper;
import com.jzlife.report.dto.ServiceGmvDto;
import com.jzlife.report.entity.hmorder.DiMerchandiseDisplay;
import com.jzlife.report.entity.hmorder.DiRServicegmvLeft;
import com.jzlife.report.entity.hmorder.DiRServicegmvRight;
import com.jzlife.report.entity.nurse.SysCity;
import com.jzlife.report.service.hmorder.ReportService;
import com.jzlife.report.service.redis.RedisService;
import com.jzlife.report.util.AccountDate;
import com.jzlife.report.util.JsonUtil;
import com.jzlife.report.vo.PersonWorkAvgVo;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;

import java.math.BigDecimal;
import java.math.RoundingMode;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;

/**
 * @author fuxubing
 */
@Service
public class ReportServiceImpl implements ReportService {
	@Resource
	private SalesReportMapper salesReportMapper;
	@Resource
	private DiMerchandiseDisplayMapper diMerchandiseDisplayMapper;

	@Autowired
	private DiRServicegmvLeftMapper diRServicegmvLeftMapper;

	@Autowired
	private DiRServicegmvRightMapper diRServicegmvRightMapper;

	@Resource
	private RedisService redisService;

	@Autowired
	private SysCityMapper cityMapper;

	public static String REPORT_SERVICE_GMV_LEFT_KEY = "REPORT_SGMV_LEFT_KEY";
	public static String REPORT_SERVICE_GMV_RIGHT_KEY = "REPORT_SGMV_RIGHT_KEY";

	public static String TABLE_HEAD_BAOJIEWUSHIFU = "保洁-无实付";
	public static String TABLE_HEAD_BAOJIEHEJI = "保洁-合计";
	public static String TABLE_HEAD_QUANYEWUXIANHEJI = "全业务线合计";

	@Override
	public Map<String, Object> getSaleReportData(String startTime, String endTime, String city, String storeNames,
			String businessLineStr, String levelOnes, String levelTwos) {
		// 报表返回数据封装
		List<PersonWorkAvgVo> pwaList = Lists.newArrayList();
		// 获取每一天
		List<String> timeList = AccountDate.getEveryday(startTime, endTime);
		// 业务线
		List<String> businessLines = diMerchandiseDisplayMapper.selectDistinctBusinessLine(businessLineStr);
		// 销售数据
		List<Map<String, Object>> maps = salesReportMapper.selectSalesReportData(startTime, endTime, city, storeNames,
				businessLineStr, levelOnes, levelTwos);

		businessLines.forEach(businessLine -> {

			for (int i = 0; i < 2; i++) {
				PersonWorkAvgVo personWorkAvgVo = new PersonWorkAvgVo();
				if (i == 0) {
					personWorkAvgVo.setName(businessLine);
					personWorkAvgVo.setType("bar");
					List<String> list = new ArrayList<>();
					timeList.forEach(item -> {
						String data = "0.00";
						for (Map<String, Object> map : maps) {
							if (item.equals(map.get("date").toString())
									&& businessLine.equals(map.get("businessLine").toString())) {
								data = map.get("realFees").toString();
								break;
							}
						}
						list.add(data);
					});
					personWorkAvgVo.setData(list);
				} else {
					personWorkAvgVo = new PersonWorkAvgVo();
					personWorkAvgVo.setName(businessLine);
					personWorkAvgVo.setType("line");
					List<String> list = new ArrayList<>();
					timeList.forEach(item -> {
						String data = "0.00";
						for (Map<String, Object> map : maps) {
							if (item.equals(map.get("date").toString())
									&& businessLine.equals(map.get("businessLine").toString())) {
								data = map.get("dljs").toString();
								break;
							}
						}
						list.add(data);
					});
					personWorkAvgVo.setData(list);
				}
				pwaList.add(personWorkAvgVo);
			}
		});

		Map<String, Object> map = new HashMap<>(16);
		map.put("timeList", timeList);
		map.put("pwaList", pwaList);
		return map;
	}

	@Override
	public Map<String, Object> getMonthlySaleReportData(String beginMonth, String endMonth, String city,
			String storeNames, String businessLineStr, String levelOnes, String levelTwos) {
		// 报表返回数据封装
		List<PersonWorkAvgVo> pwaList = Lists.newArrayList();
		// 获取每月
		List<String> months = AccountDate.getAppointMonths(beginMonth, endMonth);
		// 业务线
		List<String> businessLines = diMerchandiseDisplayMapper.selectDistinctBusinessLine(businessLineStr);
		// 销售数据
		List<Map<String, Object>> maps = salesReportMapper.selectMonthLySalesReportData(beginMonth, endMonth, city,
				storeNames, businessLineStr, levelOnes, levelTwos);

		businessLines.forEach(businessLine -> {

			for (int i = 0; i < 2; i++) {
				PersonWorkAvgVo personWorkAvgVo = new PersonWorkAvgVo();
				if (i == 0) {
					personWorkAvgVo.setName(businessLine);
					personWorkAvgVo.setType("bar");
					List<String> list = new ArrayList<>();
					months.forEach(item -> {
						String data = "0.00";
						for (Map<String, Object> map : maps) {
							if (item.equals(map.get("yearmonth").toString())
									&& businessLine.equals(map.get("businessLine").toString())) {
								data = map.get("realFees").toString();
								break;
							}
						}
						list.add(data);
					});
					personWorkAvgVo.setData(list);
				} else {
					personWorkAvgVo = new PersonWorkAvgVo();
					personWorkAvgVo.setName(businessLine);
					personWorkAvgVo.setType("line");
					List<String> list = new ArrayList<>();
					months.forEach(item -> {
						String data = "0.00";
						for (Map<String, Object> map : maps) {
							if (item.equals(map.get("yearmonth").toString())
									&& businessLine.equals(map.get("businessLine").toString())) {
								data = map.get("dljs").toString();
								break;
							}
						}
						list.add(data);
					});
					personWorkAvgVo.setData(list);
				}
				pwaList.add(personWorkAvgVo);
			}
		});

		Map<String, Object> map = new HashMap<>(16);
		map.put("months", months);
		map.put("pwaList", pwaList);
		return map;
	}

	public Map<Object, Object> getGmvData(String startTime, String endTime) {
		List<DiRServicegmvLeft> left = getLeft(startTime, endTime);// 服务

		List<DiRServicegmvRight> right = getRight(startTime, endTime);// 销售 实际业务处理
		// 设置 无实付数量，无实付金额
		setLossCountandFee(left, right);
		// 表格头部--业务线
		List<DiMerchandiseDisplay> diList = getBusinessLineHead();
		// 城市
		List<SysCity> cityList = cityMapper.selectAll();
		SysCity sysCity = new SysCity();
		sysCity.setCityName("合计");
		cityList.add(sysCity);

		List<ServiceGmvDto> listGmv = new ArrayList<ServiceGmvDto>();
		for (SysCity s : cityList) {
			// 每次加入单量和金额
			ServiceGmvDto dl = new ServiceGmvDto();
			dl.setCity(s.getCityName());
			dl.setName("单量");

			ServiceGmvDto je = new ServiceGmvDto();
			je.setCity(s.getCityName());
			je.setName("金额");

			Map<Object, Object> orderSum = new HashMap<Object, Object>();// 单量
			Map<Object, Object> amount = new HashMap<Object, Object>();// 金额
			for (DiMerchandiseDisplay di : diList) {
				if (di.getAttr1() == null) {
					continue;
				}

				if (!s.getCityName().equals("合计")) {
					// Attr1 在 表格头部--业务线 处理
					orderSum.put(di.getAttr1(), getorderSum(di, s, right));
					amount.put(di.getAttr1(), getAmount(di, s, right));
				}

				if (s.getCityName().equals("合计")) {
					// 计算 合计的总单量，总金额
					orderSum.put(di.getAttr1(), calculationAll(listGmv, "单量", di.getAttr1()));
					amount.put(di.getAttr1(), calculationAll(listGmv, "金额", di.getAttr1()));
				}
			}
			// 保洁-合计 ，全业务线合计
			heji(orderSum, amount,s.getCityName());//这里在上面计算了  if (s.getCityName().equals("合计")) {

			dl.setSumOramount(orderSum);
			je.setSumOramount(amount);

			listGmv.add(dl);
			listGmv.add(je);
		}

		Map<Object, Object> data = new HashMap<Object, Object>();
		data.put("diList", diList);
		data.put("listGmv", listGmv);

		return data;
	}

	/**
	 *
	 * @param listGmv 所有的GMV数据
	 * @param name    单量或金额
	 * @param key     表格的头部key
	 * @return
	 */
	private BigDecimal calculationAll(List<ServiceGmvDto> listGmv, String name, String key) {
		BigDecimal big = new BigDecimal(0);
		for (ServiceGmvDto g : listGmv) {
			Map<Object, Object> sumOramount = g.getSumOramount();
			if (g.getName().equals(name)) {
				for (Entry<Object, Object> t : sumOramount.entrySet()) {
					if (t.getKey().equals(key)) {
						big = big.add(new BigDecimal(t.getValue().toString()));
					}
				}
			}
		}
		return big;
	}

	/**
	 * 保洁-合计 ，全业务线合计
	 *
	 * @param dl       单量 对象
	 * @param je       金额 对象
	 * @param orderSum 单量行数据
	 * @param amount   金额行数据
	 */
	private void heji(Map<Object, Object> orderSum, Map<Object, Object> amount,String cityName) {
		if(cityName.equals("合计")) {
			return;
		}

		// 保洁-合计 ，全业务线合计 (单量，金额)
		BigDecimal hjdl = new BigDecimal(0);// 保洁合计-单量
		BigDecimal qywx_dl = new BigDecimal(0);// 全业务线合计-单量

		for (Entry<Object, Object> m : orderSum.entrySet()) {
			if (m.getKey().toString().contains("保洁")) {
				hjdl = hjdl.add(new BigDecimal(m.getValue().toString()));
			} else {
				qywx_dl = qywx_dl.add(new BigDecimal(m.getValue().toString()));
			}
		}
		orderSum.put(TABLE_HEAD_BAOJIEHEJI, hjdl);
		orderSum.put(TABLE_HEAD_QUANYEWUXIANHEJI, qywx_dl.add(hjdl));

		BigDecimal hjje = new BigDecimal(0);// 保洁合计-金额
		BigDecimal qywx_je = new BigDecimal(0);// 全业务线合计-金额
		for (Entry<Object, Object> m : amount.entrySet()) {
			if (m.getKey().toString().contains("保洁")) {
				hjje = hjje.add(new BigDecimal(m.getValue().toString()));
			} else {
				qywx_je = qywx_je.add(new BigDecimal(m.getValue().toString()));
			}
		}
		amount.put(TABLE_HEAD_BAOJIEHEJI, hjje);
		amount.put(TABLE_HEAD_QUANYEWUXIANHEJI, qywx_je.add(hjje));
	}

	public BigDecimal getAmount(DiMerchandiseDisplay di, SysCity s, List<DiRServicegmvRight> right) {
		BigDecimal big = new BigDecimal(0);

		if (di.getAttr1().equals(TABLE_HEAD_BAOJIEWUSHIFU)) {
			for (DiRServicegmvRight r : right) {
				if (r.getCity().equals(s.getCityName()) && r.getBusinessLine().equals(di.getBusinessLine())
						&& null != r.getLossFee()) {
					big = big.add(new BigDecimal(r.getLossFee()));
				}
			}
			return big;
		}

//		if (di.getAttr1().equals("全业务线合计")) {
//
//			return null;
//		}

		for (DiRServicegmvRight r : right) {

			if (r.getBusinessLine().equals("保洁")) {
				String str = r.getBusinessLine() + "-" + (r.getStinct().equals("-") ? r.getLevelOne() : r.getStinct());
				if (r.getCity().equals(s.getCityName()) && str.equals(di.getAttr1()) && null != r.getPayfee()) {
					big = big.add(r.getPayfee());
				}

			} else {
				if (r.getCity().equals(s.getCityName()) && r.getBusinessLine().equals(di.getBusinessLine())
						&& r.getLevelOne().equals(di.getLevelOne()) && null != r.getCount()) {
					big = big.add(r.getPayfee());
				}
			}
		}

		return big;
	}

	/**
	 * 单量
	 *
	 * @param di
	 * @param s
	 * @param right
	 * @return
	 */
	public Integer getorderSum(DiMerchandiseDisplay di, SysCity s, List<DiRServicegmvRight> right) {
		int sums = 0;

		if (di.getAttr1().equals(TABLE_HEAD_BAOJIEWUSHIFU)) {
			for (DiRServicegmvRight r : right) {
				if (r.getCity().equals(s.getCityName()) && r.getBusinessLine().equals(di.getBusinessLine())
						&& null != r.getLossCount()) {
					sums = sums + Integer.parseInt(r.getLossCount());
				}
			}
			return sums;
		}

//		if (di.getAttr1().equals("全业务线合计")) {
//
//			return null;
//		}

		for (DiRServicegmvRight r : right) {

			if (r.getBusinessLine().equals("保洁")) {
				String str = r.getBusinessLine() + "-" + (r.getStinct().equals("-") ? r.getLevelOne() : r.getStinct());
				if (r.getCity().equals(s.getCityName()) && str.equals(di.getAttr1()) && null != r.getCount()) {
					sums = sums + r.getCount().intValue();
				}

			} else {
				if (r.getCity().equals(s.getCityName()) && r.getBusinessLine().equals(di.getBusinessLine())
						&& r.getLevelOne().equals(di.getLevelOne()) && null != r.getCount()) {
					sums = sums + r.getCount().intValue();
				}
			}
		}

		return sums;
	}

	/**
	 * 设置 无实付数量，无实付金额
	 *
	 * @param left
	 * @param right
	 */
	private void setLossCountandFee(List<DiRServicegmvLeft> left, List<DiRServicegmvRight> right) {
		for (DiRServicegmvRight l : right) {
			for (DiRServicegmvLeft r : left) {
				if (l.getBeginDate().equals(r.getBeginDate()) && l.getCity().equals(r.getCity())
						&& l.getBusinessLine().equals(r.getBusinessLine()) && l.getLevelOne().equals(r.getLevelOne())
						&& l.getLevelTwo().equals(r.getLevelTwo())) {
					l.setLossCount(r.getCount().toString());// 无实付单量
					break;
				}
			}
			if (null != l.getLossCount() && !l.getLossCount().equals("0")) {
				String lossFee = getLossFee(l, right);
				l.setLossFee(lossFee);
			}
		}
	}

	/**
	 * 表格头部
	 *
	 * @param diList
	 */
	private List<DiMerchandiseDisplay> getBusinessLineHead() {
		List<DiMerchandiseDisplay> diList = new ArrayList<DiMerchandiseDisplay>();

		List<DiMerchandiseDisplay> selectGroupType = diMerchandiseDisplayMapper.selectGroupType();
		for (DiMerchandiseDisplay d : selectGroupType) {
			if (d.getBusinessLine().equals("保洁")) {// stinct -
				d.setAttr1(d.getBusinessLine() + "-" + (d.getStinct().equals("-") ? d.getLevelOne() : d.getStinct()));
			} else {
				d.setAttr1(d.getBusinessLine() + "-" + d.getLevelOne());
			}
		}

		diList.addAll(selectGroupType);

		// 无实付，保洁合计，全业务线合计 是固定的
		DiMerchandiseDisplay t1 = new DiMerchandiseDisplay();
		t1.setBusinessLine("保洁");
		t1.setLevelOne("无实付");
		t1.setAttr1(TABLE_HEAD_BAOJIEWUSHIFU);

		DiMerchandiseDisplay t2 = new DiMerchandiseDisplay();
		t2.setBusinessLine("保洁");
		t2.setLevelOne("合计");
		t2.setAttr1(TABLE_HEAD_BAOJIEHEJI);

		DiMerchandiseDisplay t3 = new DiMerchandiseDisplay();
		t3.setBusinessLine("全业务线");
		t3.setLevelOne("合计");
		t3.setAttr1(TABLE_HEAD_QUANYEWUXIANHEJI);

		diList.add(t1);
		diList.add(t2);
		diList.add(t3);

		Collections.sort(diList, new Comparator<DiMerchandiseDisplay>() {

			@Override
			public int compare(DiMerchandiseDisplay o1, DiMerchandiseDisplay o2) {
				if (o1.getAttr1().contains("保洁")) {
					return 1;
				} else {
					if (o1.getAttr1().contains("全业务线合计")) {
						return 1;
					} else {
						return -1;
					}
				}

			}

		});

		return diList;
	}

	/**
	 * 计算无实付金额
	 *
	 * @param current
	 * @param list
	 * @return
	 */
	public String getLossFee(DiRServicegmvRight current, List<DiRServicegmvRight> list) {
		// 无实付金额
		// 根据
		BigDecimal average = new BigDecimal(0);
		BigDecimal ct = new BigDecimal(0);
		BigDecimal one = new BigDecimal(1);
		for (DiRServicegmvRight d : list) {
			if (current.getCity().equals(d.getCity()) && current.getBusinessLine().equals(d.getBusinessLine())
					&& current.getLevelOne().equals(d.getLevelOne()) && current.getLevelTwo().equals(d.getLevelTwo())) {
				average = average.add(d.getAverage());
				ct = ct.add(one);
			}
		}
		// 均价
		BigDecimal avg = average.divide(ct, 3, RoundingMode.HALF_UP);// 保留3位 average.divide(ct, 3,
																		// RoundingMode.HALF_UP);
		// 无实付
		BigDecimal lossfee = avg.multiply(new BigDecimal(current.getLossCount()));

		return lossfee.toString();
	}

	public List<DiRServicegmvLeft> getLeft(String startTime, String endTime) {
		String key = startTime + endTime;
		key = REPORT_SERVICE_GMV_LEFT_KEY + ":" + key.replaceAll(":", "");

		String leftData = redisService.get(key);
		List<DiRServicegmvLeft> left = new ArrayList<DiRServicegmvLeft>();
		if (StringUtils.isNotBlank(leftData)) {
			left = JsonUtil.jsonToList(leftData, DiRServicegmvLeft.class);
		} else {
			left = diRServicegmvLeftMapper.selectByProcedure(startTime, endTime);
			if (null != left && left.size() > 0) {
				redisService.set(key, JsonUtil.getJson(left));
				redisService.expire(key, 30 * 60);
			}
		}
		return left;
	}

	public List<DiRServicegmvRight> getRight(String startTime, String endTime) {
		String key = startTime + endTime;
		key = REPORT_SERVICE_GMV_RIGHT_KEY + ":" + key.replaceAll(":", "");

		String leftData = redisService.get(key);
		List<DiRServicegmvRight> right = new ArrayList<DiRServicegmvRight>();
		if (StringUtils.isNotBlank(leftData)) {
			right = JsonUtil.jsonToList(leftData, DiRServicegmvRight.class);
		} else {
			right = diRServicegmvRightMapper.selectByProcedure(startTime, endTime);
			if (null != right && right.size() > 0) {
				redisService.set(key, JsonUtil.getJson(right));
				redisService.expire(key, 30 * 60);
			}
		}
		return right;
	}

}
