package com.jzlife.report.service.newreport.impl;

import com.jzlife.report.dao.nurse.ShopReferenceMapper;
import com.jzlife.report.service.newreport.ShopReferenceService;
import com.jzlife.report.vo.ReportShopTrendAndProductsVo;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.math.BigDecimal;
import java.time.LocalDate;
import java.time.format.DateTimeFormatter;
import java.time.temporal.TemporalAdjusters;
import java.util.*;
import java.util.stream.Collectors;


@Service
@Slf4j
public class ShopReferenceServiceImpl implements ShopReferenceService {


    @Autowired
    private ShopReferenceMapper shopReferenceMapper;

    private static Map<String, String> cityMap = new HashMap<>(16);

    static {
        cityMap.put("上海", "2");
        cityMap.put("北京", "3");
        cityMap.put("深圳", "4");
        cityMap.put("南京", "32");
        cityMap.put("苏州", "33");
        cityMap.put("无锡", "34");
        cityMap.put("杭州", "35");
        cityMap.put("青岛", "36");
        cityMap.put("成都", "39");
        cityMap.put("重庆", "38");
        cityMap.put("广州", "40");
    }


    @Override

    public Map<String, Object> shopReferenceGoal(String month, String userId, String city) {
        String cityIds;
        if (StringUtils.contains(city, ",")) {
            cityIds = Arrays.stream(city.split(",")).map(cityMap::get).distinct().collect(Collectors.joining(","));
        } else {
            cityIds = cityMap.get(city);
        }

        Map<String, Object> hash = new HashMap<>();
        String startTime = LocalDate.now().withMonth(Integer.parseInt(month.split("-")[1])).with(TemporalAdjusters.firstDayOfMonth()).format(DateTimeFormatter.ofPattern("yyyy-MM-dd"));
        String endTime = LocalDate.now().withMonth(Integer.parseInt(month.split("-")[1])).with(TemporalAdjusters.lastDayOfMonth()).format(DateTimeFormatter.ofPattern("yyyy-MM-dd"));
        List<List<?>> lists = shopReferenceMapper.callResult(startTime, endTime, cityIds);
        List<?> list = lists.get(0);
        hash.put("resultIndex", list);
        return hash;
    }

    @Override
    public Map<String, Object> newAdditionsAndSalesTrendsAndHighestSalesProducts(ReportShopTrendAndProductsVo vo, String userId) {
        HashMap<String, Object> result = new HashMap<>();
        // 走势图 分 天和月
        String cityIds = vo.getCity();
        if (StringUtils.contains(vo.getCity(), ",")) {
            cityIds = Arrays.stream(vo.getCity().split(",")).distinct().map(cityMap::get).collect(Collectors.joining(","));
        } else {
            cityIds = cityMap.get(cityIds);
        }
        if (vo.getTimeType() == 1) {
            // 月
            String start = LocalDate.now().withYear(Integer.parseInt(vo.getStartMonth().split("-")[0])).withMonth(Integer.parseInt(vo.getStartMonth().split("-")[1])).with(TemporalAdjusters.firstDayOfMonth()).format(DateTimeFormatter.ofPattern("yyyy-MM-dd"));
            String end = LocalDate.now().withYear(Integer.parseInt(vo.getEndMonth().split("-")[0])).withMonth(Integer.parseInt(vo.getEndMonth().split("-")[1])).with(TemporalAdjusters.lastDayOfMonth()).format(DateTimeFormatter.ofPattern("yyyy-MM-dd"));
            List<List<?>> lists = shopReferenceMapper.callResult(start, end, cityIds);
            parseResultList(lists, result, 1);
        } else {
            List<List<?>> lists = shopReferenceMapper.callResult(vo.getStartTime(), vo.getEndTime(), cityIds);
            parseResultList(lists, result, 2);
        }
        //当月销量最多的商品
        String startTime = LocalDate.now().with(TemporalAdjusters.firstDayOfMonth()).format(DateTimeFormatter.ofPattern("yyyy-MM-dd"));
        String endTime = LocalDate.now().with(TemporalAdjusters.lastDayOfMonth()).format(DateTimeFormatter.ofPattern("yyyy-MM-dd"));
        List<List<?>> lists = shopReferenceMapper.callResult(startTime, endTime, cityIds);
        List<?> list = lists.get(2);
        result.put("goods", list);
        return result;
    }

    private void parseResultList(List<List<?>> lists, HashMap<String, Object> result, int type) {
        List<Map<String, Object>> list = (List<Map<String, Object>>) lists.get(1);
        if (type == 2) {
            List<String> dates = list.stream().map(a -> (String) a.get("日期")).collect(Collectors.toList());
            result.put("日期", dates);
            List<BigDecimal> zIncrease = list.stream().map(a -> (BigDecimal) a.get("z_increase")).collect(Collectors.toList());
            result.put("直客", zIncrease);
            List<BigDecimal> hyIncrease = list.stream().map(a -> (BigDecimal) a.get("hy_increase")).collect(Collectors.toList());
            result.put("会员", hyIncrease);
            List<BigDecimal> yjIncrease = list.stream().map(a -> (BigDecimal) a.get("yj_increase")).collect(Collectors.toList());
            result.put("引荐人新增", yjIncrease);
            List<BigDecimal> xsAmount = list.stream().map(a -> (BigDecimal) a.get("xs_amount")).collect(Collectors.toList());
            result.put("销售额", xsAmount);
        } else {
            // 处理月
            Set<String> months = new HashSet<>();
            for (Map<String, Object> objectMap : list) {
                String yearMonth = ((String) objectMap.get("日期")).substring(0, 7);
                months.add(yearMonth);
            }
            List<String> monthList = new ArrayList<>(months);
            monthList.sort(String::compareTo);
            result.put("月份", monthList);
            List<BigDecimal> zList = new ArrayList<>();
            List<BigDecimal> hList = new ArrayList<>();
            List<BigDecimal> yjList = new ArrayList<>();
            List<BigDecimal> xsAmont = new ArrayList<>();
            for (String month : monthList) {
                BigDecimal zInc = new BigDecimal(0);
                BigDecimal hInc = new BigDecimal(0);
                BigDecimal yjInc = new BigDecimal(0);
                BigDecimal xsAmount = new BigDecimal(0);
                for (Map<String, Object> objectMap : list) {
                    String yearMonth = ((String) objectMap.get("日期")).substring(0, 7);
                    if (month.equals(yearMonth)) {
                        zInc = zInc.add(objectMap.get("z_increase") == null ? BigDecimal.ZERO : (BigDecimal) objectMap.get("z_increase"));
                        hInc = hInc.add(objectMap.get("hy_increase") == null ? BigDecimal.ZERO : (BigDecimal) objectMap.get("hy_increase"));
                        yjInc = yjInc.add(objectMap.get("yj_increase") == null ? BigDecimal.ZERO : (BigDecimal) objectMap.get("yj_increase"));
                        xsAmount = xsAmount.add(objectMap.get("xs_amount") == null ? BigDecimal.ZERO : (BigDecimal) objectMap.get("xs_amount"));
                    }
                }
                zList.add(zInc);
                hList.add(hInc);
                yjList.add(yjInc);
                xsAmont.add(xsAmount);
            }
            result.put("直客", zList);
            result.put("会员", hList);
            result.put("引荐人新增", yjList);
            result.put("销售额", xsAmont);
        }
    }
}
