package com.jzlife.report.service.report.impl;

import java.util.List;

import com.jzlife.report.dto.ReportProductStocksVo;
import org.apache.ibatis.annotations.Param;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.jzlife.report.dao.reports.BjStockStatisticsMapper;
import com.jzlife.report.dao.reports.HmStationStockAvailabilityMapper;
import com.jzlife.report.dto.export.BuzlineStockAvailabilityExportDto;
import com.jzlife.report.dto.export.CityBuzServiceorderNumberExport;
import com.jzlife.report.dto.export.CityStationCustomerComplaintExport;
import com.jzlife.report.dto.export.StockStationPersonExportDto;
import com.jzlife.report.entity.reports.HmStationStockAvailability;
import com.jzlife.report.service.report.BjStockStatisticsService;

@Service
public class BjStockStatisticsServiceImpl implements BjStockStatisticsService {

	@Autowired
	BjStockStatisticsMapper bjStockStatisticsMapper;

	@Autowired
	HmStationStockAvailabilityMapper availabilityMapper;

	public List<StockStationPersonExportDto> getStockStationPerson(Boolean isExport, String ofMonth) {
		return bjStockStatisticsMapper.sp_stock_station_person(ofMonth);
	}

	public List<CityStationCustomerComplaintExport> getCityBuzlineServiceorderNumber(String date) {
		return bjStockStatisticsMapper.sp_city_buzline_serviceorder_number(date);
	}

	public List<BuzlineStockAvailabilityExportDto> buzlineStockAvailability(String ofMonth) {
		return bjStockStatisticsMapper.sp_report_buzline_stock_availability(ofMonth);
	}

	public List<CityBuzServiceorderNumberExport> getCityBuzServiceorderNumber(String dateStart, String dateEnd) {
		return bjStockStatisticsMapper.sp_city_station_customer_complaint(dateStart, dateEnd);
	}

	@Override
	@Transactional
	public void importHmStockAvailability(List<HmStationStockAvailability> cityStock,
			List<HmStationStockAvailability> stationStock, String ofMonth) {

		availabilityMapper.deleteOfMonth(ofMonth);

		if (null != cityStock && cityStock.size() > 0) {
			cityStock.forEach(e->{
				e.setCityShort(e.getCity().replace("市", ""));
				e.setStockMonthRatio(e.getStockMonthRatio().contains("-")?null:e.getStockMonthRatio());
				e.setType(2);//1服务站库存率，2城市库存率
				availabilityMapper.insertSelective(e);
			});
		}

		if (null != stationStock && stationStock.size() > 0) {
			stationStock.forEach(e->{
				if(e.getCity()!=null) {
					e.setCityShort(e.getCity().replace("市", ""));
					e.setStockMonthRatio(e.getStockMonthRatio().contains("-")?null:e.getStockMonthRatio());
					e.setType(1);//1服务站库存率，2城市库存率
					availabilityMapper.insertSelective(e);
				}
			});
		}
		
		//处理结果到report_city_avgorder_workeinghoure
		availabilityMapper.sp_report_city_avgorder_workeinghoure(ofMonth);

	}

	@Override
	public  List<ReportProductStocksVo> getCityProductStock(String city){
		return bjStockStatisticsMapper.get_city_product_stock(city);
	}
}
