package com.jzlife.report.service.report.impl;

import java.awt.image.RescaleOp;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import com.jzlife.report.dao.reports.ReportPrivateDomainMapper;
import com.jzlife.report.dto.privatedomain.GoodsSalesInfo;
import com.jzlife.report.dto.privatedomain.goodsCitySalesInfo;
import com.jzlife.report.dto.privatedomain.RankDto;
import com.jzlife.report.dto.privatedomain.RankDto.Sub;
import com.jzlife.report.service.report.ReportPrivateDomainService;
import com.jzlife.report.util.DateUtil;
import com.jzlife.report.util.ResponseUtil;

@Service
public class ReportPrivateDomainServiceImpl implements ReportPrivateDomainService {

	@Autowired
	ReportPrivateDomainMapper domainMapper;

	public Object getAllRank(Integer type, Integer dayOrMonth) {
		// type 固定查某类 limit是999 ，dayOrMonth查某天或某月 1天，2月
		Integer limit = type != null ? 999 : 3;
		String day = null;
		String month = null;

		if (dayOrMonth.equals(1)) {
			day = DateUtil.getBeforeDay(-1);
		} else {
			month = DateUtil.getBeforeMonth(-1);
		}

		Map<String, Object> data = new HashMap<String, Object>();
		if (type == null || type.equals(1)) {// 城市销量排行
			data.put("citySales", getCitySales(limit, day, month));
		}
		if (type == null || type.equals(2)) {// 加盟商销售排行
			data.put("jmsSales", getJmsSales(limit, day, month));
		}
		if (type == null || type.equals(3)) {// 履约排行
			data.put("performanceRank", getPerformanceRank(limit, day, month));
		}

		if (type == null || type.equals(4)) {// 商品销售排行
			data.put("goodsSalesRank", getGoodsSalesRank(limit, day, month));
		}

		if (type == null || type.equals(5)) {// poi排行
			data.put("poiRank", getPoiRank(limit, day, month));
		}

		return ResponseUtil.ok(data);
	}

	/**
	 * 商品销售排行
	 * @param limit
	 * @param day
	 * @param month
	 * @return
	 */
	private List<RankDto> getGoodsSalesRank(Integer limit, String day, String month) {
		List<RankDto> rankList = new ArrayList<RankDto>();
		List<GoodsSalesInfo> goodsSales = domainMapper.goodsSalesRank(month, day, limit);
		if (goodsSales.size() == 0) {
			return rankList;
		}
		List<Integer> goodsId=new ArrayList<Integer>();
		for (GoodsSalesInfo d : goodsSales) {
			RankDto r = new RankDto();
			r.setTempId(d.getGoodsId());
			r.setName(d.getGoodsName());
			r.setSalesNumber(d.getCountSales());
			rankList.add(r);
			goodsId.add(d.getGoodsId());
		}
		
		List<goodsCitySalesInfo> goodsCityRank = domainMapper.goodsCityRank(month, day, StringUtils.join(goodsId,","));
		List<Sub> allsub=new ArrayList<RankDto.Sub>();
		if(goodsCityRank.size()>0) {
			Map<Integer, List<goodsCitySalesInfo>> map=goodsCityRank.stream().collect(Collectors.groupingBy(goodsCitySalesInfo::getGoodsId));
		
			for(RankDto r:rankList) {
				List<goodsCitySalesInfo> list = map.get(r.getTempId());
				List<Sub> subList=new ArrayList<RankDto.Sub>();
				list.forEach(e->{
					Sub sub=new Sub();
					sub.setName(e.getCityName());
					sub.setValue(e.getCountSales());
					subList.add(sub);
				});
				r.setSubList(subList);
				allsub.addAll(subList);
			}
		}
		
		getMax(rankList, allsub);
		return rankList;
	}

	private List<RankDto> getPoiRank(Integer limit, String day, String month) {
		List<RankDto> rankList = new ArrayList<RankDto>();
		List<Map<String, Object>> salesRank = domainMapper.poiRank(month, day, limit);
		if (salesRank.size() == 0) {
			return rankList;
		}
		List<Sub> allsub = new ArrayList<RankDto.Sub>();
		for (Map<String, Object> d : salesRank) {
			RankDto r = new RankDto();
			r.setName(d.get("shopName"));
			r.setSalesNumber(d.get("shopSales"));
			r.setRate(d.get("rate"));

			List<Sub> subList = new ArrayList<RankDto.Sub>();
			Sub s = new Sub();
			s.setName("保洁");
			s.setValue(Double.valueOf(d.get("baojie").toString()));
			subList.add(s);
			Sub s2 = new Sub();
			s2.setName("品类");
			s2.setValue(Double.valueOf(d.get("pinlei").toString()));
			subList.add(s2);
			Sub s3 = new Sub();
			s3.setName("其他");
			s3.setValue(Double.valueOf(d.get("qita").toString()));
			subList.add(s3);

			r.setSubList(subList);
			rankList.add(r);

			allsub.addAll(subList);// 获取最大值
		}

		getMax(rankList, allsub);
		return rankList;
	}

	/**
	 * 履约排行
	 * 
	 * @param limit
	 * @param day
	 * @param month
	 * @return
	 */
	private List<RankDto> getPerformanceRank(Integer limit, String day, String month) {
		List<RankDto> rankList = new ArrayList<RankDto>();
		List<Map<String, Object>> salesRank = domainMapper.performanceRank(month, day, limit);
		if (salesRank.size() == 0) {
			return rankList;
		}
		List<Sub> allsub = new ArrayList<RankDto.Sub>();
		for (Map<String, Object> d : salesRank) {
			RankDto r = new RankDto();
			r.setName(d.get("fra_name"));
			r.setSalesNumber(d.get("jmsbookNumber"));
			r.setRate(d.get("aftersaleRate"));

			List<Sub> subList = new ArrayList<RankDto.Sub>();
			Sub s = new Sub();
			s.setName("分配");
			s.setValue(Double.valueOf(d.get("jmsbookNumber").toString()));
			subList.add(s);
			Sub s2 = new Sub();
			s2.setName("履约");
			s2.setValue(Double.valueOf(d.get("jmsServicedNumber").toString()));
			subList.add(s2);

			r.setSubList(subList);
			rankList.add(r);

			allsub.addAll(subList);// 获取最大值
		}

		getMax(rankList, allsub);
		return rankList;
	}

	/**
	 * 加盟商销量排行
	 * 
	 * @param limit
	 * @param day
	 * @param month
	 * @return
	 */
	private List<RankDto> getJmsSales(Integer limit, String day, String month) {
		List<RankDto> rankList = new ArrayList<RankDto>();
		List<Map<String, Object>> salesRank = domainMapper.jmsSalesRank(month, day, limit);
		if (salesRank.size() == 0) {
			return rankList;
		}
		List<Sub> allsub = new ArrayList<RankDto.Sub>();
		for (Map<String, Object> d : salesRank) {
			RankDto r = new RankDto();
			r.setName(d.get("fra_name"));
			r.setSalesNumber(d.get("jmsSales"));
			r.setRate(d.get("rate"));

			List<Sub> subList = new ArrayList<RankDto.Sub>();
			Sub s = new Sub();
			s.setName("保洁");
			s.setValue(Double.valueOf(d.get("baojie").toString()));
			subList.add(s);
			Sub s2 = new Sub();
			s2.setName("品类");
			s2.setValue(Double.valueOf(d.get("pinlei").toString()));
			subList.add(s2);
			Sub s3 = new Sub();
			s3.setName("其他");
			s3.setValue(Double.valueOf(d.get("qita").toString()));
			subList.add(s3);

			r.setSubList(subList);
			rankList.add(r);

			allsub.addAll(subList);// 获取最大值
		}

		getMax(rankList, allsub);
		return rankList;
	}

	/**
	 * 城市销量排行
	 * 
	 * @param limit
	 * @param day
	 * @param month
	 * @return
	 */
	private List<RankDto> getCitySales(Integer limit, String day, String month) {
		List<RankDto> rankList = new ArrayList<RankDto>();
		List<Map<String, Object>> citySalesRank = domainMapper.citySalesRank(month, day, limit);
		if (citySalesRank.size() == 0) {
			return rankList;
		}
		List<Sub> allsub = new ArrayList<RankDto.Sub>();
		for (Map<String, Object> d : citySalesRank) {
			RankDto r = new RankDto();
			r.setName(d.get("city"));
			r.setSalesNumber(d.get("citySales"));
			r.setRate(d.get("verifyRate"));

			List<Sub> subList = new ArrayList<RankDto.Sub>();
			Sub s = new Sub();
			s.setName("保洁");
			s.setValue(Double.valueOf(d.get("baojie").toString()));
			subList.add(s);
			Sub s2 = new Sub();
			s2.setName("品类");
			s2.setValue(Double.valueOf(d.get("pinlei").toString()));
			subList.add(s2);
			Sub s3 = new Sub();
			s3.setName("其他");
			s3.setValue(Double.valueOf(d.get("qita").toString()));
			subList.add(s3);

			r.setSubList(subList);
			rankList.add(r);

			allsub.addAll(subList);// 获取最大值
		}

		getMax(rankList, allsub);
		return rankList;
	}

	private void getMax(List<RankDto> rankList, List<Sub> allsub) {
		if (rankList.size() > 0) {
			Sub maxSub = allsub.stream().max(Comparator.comparing(Sub::getValue,Comparator.nullsLast(Double::compareTo))).get();
			rankList.forEach(f -> {
				f.getSubList().forEach(s -> {
					if(s.getValue()!=null&&maxSub.getValue()!=null&&maxSub.getValue()>0.0) {
						Double p = s.getValue() * 100 / maxSub.getValue();
						s.setProportion(p.intValue());
					}
				});
			});
		}
	}
}
