package com.jzlife.report.util;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;

import javax.net.ssl.*;
import java.io.*;
import java.net.URL;
import java.nio.charset.StandardCharsets;
import java.security.KeyManagementException;
import java.security.NoSuchAlgorithmException;
import java.security.SecureRandom;
import java.security.cert.CertificateException;
import java.security.cert.X509Certificate;
import java.util.Map;

public class HttpsUtils {
    private static final Log LOG = LogFactory.getLog(HttpsUtils.class);

    private static final class DefaultTrustManager implements X509TrustManager {
        @Override
        public void checkClientTrusted(X509Certificate[] chain, String authType) throws CertificateException {
        }

        @Override
        public void checkServerTrusted(X509Certificate[] chain, String authType) throws CertificateException {
        }

        @Override
        public X509Certificate[] getAcceptedIssuers() {
            return null;
        }
    }

    private static HttpsURLConnection getHttpsURLConnection(String uri, String method) throws IOException {
        SSLContext ctx = null;
        try {
            ctx = SSLContext.getInstance("TLS");
            ctx.init(new KeyManager[0], new TrustManager[]{new DefaultTrustManager()}, new SecureRandom());
        } catch (KeyManagementException e) {
            e.printStackTrace();
        } catch (NoSuchAlgorithmException e) {
            e.printStackTrace();
        }
        SSLSocketFactory ssf = ctx.getSocketFactory();

        URL url = new URL(uri);
        HttpsURLConnection httpsConn = (HttpsURLConnection) url.openConnection();
        httpsConn.setSSLSocketFactory(ssf);
        httpsConn.setHostnameVerifier(new HostnameVerifier() {
            @Override
            public boolean verify(String arg0, SSLSession arg1) {
                return true;
            }
        });
        httpsConn.setRequestMethod(method);
        httpsConn.setDoInput(true);
        httpsConn.setDoOutput(true);
        return httpsConn;
    }

    private static byte[] getBytesFromStream(InputStream is) throws IOException {
        ByteArrayOutputStream baos = new ByteArrayOutputStream();
        byte[] kb = new byte[1024];
        int len;
        while ((len = is.read(kb)) != -1) {
            baos.write(kb, 0, len);
        }
        byte[] bytes = baos.toByteArray();
        baos.close();
        is.close();
        return bytes;
    }

    private static String getStreamAsString(InputStream stream, String charset) throws IOException {
        try {
            Reader reader = new InputStreamReader(stream, charset);
            StringBuilder response = new StringBuilder();

            final char[] buff = new char[1024];
            int read = 0;
            while ((read = reader.read(buff)) > 0) {
                // 一次读取1024个字符
                response.append(buff, 0, read);
            }

            return response.toString();
        } finally {
            if (stream != null) {
                stream.close();
            }
        }
    }

    private static void setBytesToStream(OutputStream os, byte[] bytes) throws IOException {
        ByteArrayInputStream bais = new ByteArrayInputStream(bytes);
        byte[] kb = new byte[1024];
        int len;
        while ((len = bais.read(kb)) != -1) {
            os.write(kb, 0, len);
        }
        os.flush();
        os.close();
        bais.close();
    }

    public static String doGet(String uri) throws IOException {
        HttpsURLConnection httpsConn = getHttpsURLConnection(uri, "GET");
        return getStreamAsString(httpsConn.getInputStream(), "utf-8");
    }

    public static String doGet(String uri,String token) throws IOException {
        HttpsURLConnection httpsConn = getHttpsURLConnection(uri, "GET");
        httpsConn.setRequestProperty("access-token",token);
        return getStreamAsString(httpsConn.getInputStream(), "utf-8");
    }


    public static String doPost(String uri, String data) throws IOException {
        HttpsURLConnection httpsConn = getHttpsURLConnection(uri, "POST");
        httpsConn.setRequestProperty("Content-Type", "application/json");
        setBytesToStream(httpsConn.getOutputStream(), data.getBytes());
        return getStreamAsString(httpsConn.getInputStream(), "utf-8");
    }

    public static String doPost(String uri, String data,String token) throws IOException {
        HttpsURLConnection httpsConn = getHttpsURLConnection(uri, "POST");
        httpsConn.setRequestProperty("access-token",token);
        httpsConn.setRequestProperty("Content-Type", "application/json");
        setBytesToStream(httpsConn.getOutputStream(), data.getBytes());
        return getStreamAsString(httpsConn.getInputStream(), "utf-8");
    }
    public static String doPostFormData(String uri, String data) throws IOException {
        HttpsURLConnection httpsConn = getHttpsURLConnection(uri, "POST");
        httpsConn.setRequestProperty("Content-Type", "application/x-www-form-urlencoded");
        setBytesToStream(httpsConn.getOutputStream(), data.getBytes());
        return getStreamAsString(httpsConn.getInputStream(), "utf-8");
    }

    public static String doPost(String uri, Map<String,String> data) throws IOException {
        String boundary = "--------------------------iueriuewyrueiwriewrjj121398232132j3";
        HttpsURLConnection connection = null;
        String result = null;
        try {
            connection = getHttpsURLConnection(uri, "POST");
            connection.setRequestProperty("Content-Type", "multipart/form-data; boundary=" + boundary);
            connection.setDoOutput(true);
            connection.setDoInput(true);
            connection.setRequestProperty("Connection", "Keep-Alive");
            StringBuffer formSB = new StringBuffer();
            if (data != null) {
                if (data.size() > 0) {
                    for (Map.Entry<String, String> entry : data.entrySet()) {
                        String inputName = entry.getKey();
                        String inputValue = entry.getValue();
                        formSB.append("\r\n").append("--").append(boundary).append("\r\n");
                        formSB.append("Content-Disposition: form-data; name=\"" + inputName + "\"\r\n\r\n");
                        formSB.append(inputValue);
                    }
                    formSB.append("\r\n").append("--").append(boundary).append("--");
                }
            }
            connection.connect();
            //OutputStream out = new DataOutputStream(connection.getOutputStream());
            PrintWriter out = new PrintWriter(new OutputStreamWriter(connection.getOutputStream(), StandardCharsets.UTF_8));
            out.print(formSB);
            out.flush();
            //获得响应状态
            int resultCode = connection.getResponseCode();
            if (HttpsURLConnection.HTTP_OK == resultCode) {
                formSB = new StringBuffer();
                String readLine;
                BufferedReader responseReader = new BufferedReader(new InputStreamReader(connection.getInputStream(), StandardCharsets.UTF_8));
                while ((readLine = responseReader.readLine()) != null) {
                    formSB.append(readLine).append("\n");
                }
                responseReader.close();
                result = formSB.toString();
            } else {
                result = "{\"code\":\"" + resultCode + "\"}";
            }
            out.close();

        } catch (Exception e) {
            LOG.error(e.toString());
        } finally {
            if (connection != null)
                connection.disconnect();
        }

        return result;
    }
}
